"""
DockerVolumesDir - command ``ls -lanR /var/lib/docker/volumes/``
================================================================

A standard directory listing parser using the FileListing parser class.

Given a listing of::

    /var/lib/docker/volumes/:
    total 4
    drwx------. 3 0 0   77 Mar 15 10:50 .
    drwx-----x. 9 0 0 4096 Nov 18 22:04 ..
    drwxr-xr-x. 3 0 0   18 Mar 15 10:50 7f9d945c3b3352308a44878a5da9e6046d63e34fafbac36486f4b94f5d372b61

    /var/lib/docker/volumes/7f9d945c3b3352308a44878a5da9e6046d63e34fafbac36486f4b94f5d372b61:
    total 0
    drwxr-xr-x. 3 0 0 18 Mar 15 10:50 .
    drwx------. 3 0 0 77 Mar 15 10:50 ..
    drwxr-xr-x. 2 0 0  6 Mar 15 10:50 _data

    /var/lib/docker/volumes/7f9d945c3b3352308a44878a5da9e6046d63e34fafbac36486f4b94f5d372b61/_data:
    total 0
    drwxr-xr-x. 2 0 0  6 Mar 15 10:50 .
    drwxr-xr-x. 3 0 0 18 Mar 15 10:50 ..


Examples:

    >>> docker_dirs = shared[DockerVolumesDir]
    >>> '/var/lib/docker/volumes' in docker_dirs
    True
    >>> docker_dirs.dirs_of('/var/lib/docker/volumes')
    ['.', '..', '97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07']
    >>> '/var/lib/docker/volumes/97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07' in docker_dirs
    True
    >>> docker_dirs.dirs_of('/var/lib/docker/volumes/97d7cd1a5d8fd7730e83bb61ecbc993742438e966ac5c11910776b5d53f4ae07')
    ['.', '..', '_data']
"""

from .. import FileListing, parser, CommandParser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ls_docker_volumes)
class DockerVolumesDir(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlanR` instead.

    Read the directory for the docker volumes.
    """
    def __init__(self, *args, **kwargs):
        deprecated(DockerVolumesDir, "Please use the :class:`insights.parsers.ls.LSlanR` instead.", "3.5.0")
        super(DockerVolumesDir, self).__init__(*args, **kwargs)



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import setup_named_chroot
from insights.parsers.setup_named_chroot import SetupNamedChroot
from insights.tests import context_wrap

CHROOT_CONTENT_FILTERED = """
#!/bin/bash
# it MUST be listed last. (/var/named contains /var/named/chroot)
ROOTDIR_MOUNT='/etc/localtime /etc/named /etc/pki/dnssec-keys /etc/named.root.key /etc/named.conf
/etc/named.dnssec.keys /etc/named.rfc1912.zones /etc/rndc.conf /etc/rndc.key /usr/lib64/bind
/usr/lib/bind /etc/named.iscdlv.key /run/named /var/named /etc/protocols /etc/services'
    for all in $ROOTDIR_MOUNT; do
    for all in $ROOTDIR_MOUNT; do,
    # Check if file is mount target. Do not use /proc/mounts because detecting
""".strip()

CHROOT_CONTENT_ALL = """
#!/bin/bash

# Warning: the order is important
# If a directory containing $ROOTDIR is listed here,
# it MUST be listed last. (/var/named contains /var/named/chroot)
ROOTDIR_MOUNT='/etc/localtime /etc/named /etc/pki/dnssec-keys /etc/named.root.key /etc/named.conf
/etc/named.dnssec.keys /etc/named.rfc1912.zones /etc/rndc.conf /etc/rndc.key /etc/named.iscdlv.key /etc/protocols /etc/services
/usr/lib64/bind /usr/lib/bind /run/named
/var/named'

usage()
{
  echo
  echo 'This script setups chroot environment for BIND'
  echo 'Usage: setup-named-chroot.sh ROOTDIR [on|off]'
}

if ! [ "$#" -eq 2 ]; then
  echo 'Wrong number of arguments'
  usage
  exit 1
fi

ROOTDIR="$1"

# Exit if ROOTDIR doesn't exist
if ! [ -d "$ROOTDIR" ]; then
  echo "Root directory $ROOTDIR doesn't exist"
  usage
  exit 1
fi

mount_chroot_conf()
{
  if [ -n "$ROOTDIR" ]; then
    for all in $ROOTDIR_MOUNT; do
      # Skip nonexistant files
      [ -e "$all" ] || continue

      # If mount source is a file
      if ! [ -d "$all" ]; then
        # mount it only if it is not present in chroot or it is empty
        if ! [ -e "$ROOTDIR$all" ] || [ `stat -c'%s' "$ROOTDIR$all"` -eq 0 ]; then
          touch "$ROOTDIR$all"
          mount --bind "$all" "$ROOTDIR$all"
        fi
      else
        # Mount source is a directory. Mount it only if directory in chroot is
        # empty.
        if [ -e "$all" ] && [ `ls -1A $ROOTDIR$all | wc -l` -eq 0 ]; then
          mount --bind --make-private "$all" "$ROOTDIR$all"
        fi
      fi
    done
  fi
}

umount_chroot_conf()
{
  if [ -n "$ROOTDIR" ]; then
    for all in $ROOTDIR_MOUNT; do
      # Check if file is mount target. Do not use /proc/mounts because detecting
      # of modified mounted files can fail.
      if mount | grep -q '.* on '"$ROOTDIR$all"' .*'; then
        umount "$ROOTDIR$all"
        # Remove temporary created files
        [ -f "$all" ] && rm -f "$ROOTDIR$all"
      fi
    done
  fi
}

case "$2" in
  on)
    mount_chroot_conf
    ;;
  off)
    umount_chroot_conf
    ;;
  *)
    echo 'Second argument has to be "on" or "off"'
    usage
    exit 1
esac

exit 0
""".strip()

EXCEPTION1 = """
""".strip()

EXCEPTION2 = """
usage()
{
  echo
  echo 'This script setups chroot environment for BIND'
  echo 'Usage: setup-named-chroot.sh ROOTDIR [on|off]'
}

if ! [ "$#" -eq 2 ]; then
  echo 'Wrong number of arguments'
  usage
  exit 1
fi
""".strip()


def test_setup_named_chroot_all():
    snc = SetupNamedChroot(context_wrap(CHROOT_CONTENT_ALL))
    assert snc["ROOTDIR_MOUNT"][-1] == "/var/named"
    assert len(snc) == 2


def test_setup_named_chroot_filtered():
    snc = SetupNamedChroot(context_wrap(CHROOT_CONTENT_FILTERED))
    assert "ROOTDIR_MOUNT" in snc
    assert snc["ROOTDIR_MOUNT"][-1] != "/var/named"
    assert len(snc.raw) == 5


def test_doc_examples():
    env = {
            'snc': SetupNamedChroot(context_wrap(CHROOT_CONTENT_ALL)),
          }
    failed, total = doctest.testmod(setup_named_chroot, globs=env)
    assert failed == 0


def test_setup_named_chroot_exception1():
    with pytest.raises(SkipComponent) as e:
        SetupNamedChroot(context_wrap(EXCEPTION1))
    assert "Empty file" in str(e)


def test_setup_named_chroot_exception2():
    with pytest.raises(SkipComponent) as e:
        SetupNamedChroot(context_wrap(EXCEPTION2))
    assert "Input content is not empty but there is no useful parsed data." in str(e)



"""
xfs_quota Commands
==================

Parser contains in this module is:

XFSQuotaState - command ``/sbin/xfs_quota -x -c 'state -gu'``
-------------------------------------------------------------
"""

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.xfs_quota_state)
class XFSQuotaState(CommandParser):
    """
    The ``/sbin/xfs_quota -x -c 'state -gu'`` command provides information for the
    xfs quota devices.

    Attributes:
        group_quota (dict): Dictionary of information for each group quota
        user_quota (dict): Dictionary of information for each user quota

    Sample directory list collected::

        User quota state on /sdd (/dev/sdd)
          Accounting: ON
          Enforcement: ON
          Inode: #131 (1 blocks, 1 extents)
        Blocks grace time: [7 days]
        Blocks max warnings: 5
        Inodes grace time: [7 days]
        Inodes max warnings: 5
        Realtime Blocks grace time: [7 days]
        Group quota state on /sdd (/dev/sdd)
          Accounting: OFF
          Enforcement: OFF
          Inode: N/A
        Blocks grace time: [--------]
        Blocks max warnings: 0
        Inodes grace time: [--------]
        Inodes max warnings: 0
        Realtime Blocks grace time: [--------]
        User quota state on /sdb (/dev/sdb)
          Accounting: ON
          Enforcement: ON
          Inode: #131 (2 blocks, 2 extents)
        Blocks grace time: [6 days]
        Blocks max warnings: 10
        Inodes grace time: [6 days]
        Inodes max warnings: 5
        Realtime Blocks grace time: [6 days]
        Group quota state on /sdb (/dev/sdb)
          Accounting: ON
          Enforcement: ON
          Inode: #137 (2 blocks, 2 extents)
        Blocks grace time: [7 days]
        Blocks max warnings: 5
        Inodes grace time: [7 days]
        Inodes max warnings: 5
        Realtime Blocks grace time: [7 days]

    Examples:
        >>> type(quota_state)
        <class 'insights.parsers.xfs_quota.XFSQuotaState'>
        >>> len(quota_state.user_quota)
        3
        >>> quota_state.user_quota['/dev/sdd'] == {'device': '/dev/sdd', 'accounting': 'ON', 'enforcement': 'ON', 'inode': '#131 (1 blocks, 1 extents)', 'blocks_grace_time': '7 days', 'blocks_max_warnings': '5', 'inodes_grace_time': '7 days', 'inodes_max_warnings': '5', 'realtime_blocks_grace_time': '7 days'}
        True
    """

    def parse_content(self, content):
        self.user_quota = {}
        self.group_quota = {}
        data = None
        device = None
        for line in content:
            if not line.strip():
                continue

            if 'quota state on' in line:
                device = line.split()[-1].lstrip('(').rstrip(')')
                data = self.group_quota
                if 'User ' in line:
                    data = self.user_quota
                data[device] = data.get(device, dict(device=device))

            elif ': ' in line and device:
                key, value = line.split(':', 1)
                data[device]['_'.join(key.strip().lower().split())] = value.strip().lstrip('[').rstrip(']') if '-' not in value else None

        if not self.user_quota and not self.group_quota:
            raise SkipComponent("Empty result")



"""
CNIPodmanBridgeConf - file ``/etc/cni/net.d/87-podman-bridge.conflist``
=======================================================================

This parser converts file ``/etc/cni/net.d/87-podman-bridge.conflist``
into a dictionary that matches the JSON string in the file.

Sample file content::

    {
        "cniVersion": "0.4.0",
        "name": "podman",
        "plugins": [
            {
                "type": "bridge",
                "bridge": "cni-podman0",
                "isGateway": true,
                "ipMasq": true,
                "ipam": {
                    "type": "host-local",
                    "routes": [
                        {
                            "dst": "0.0.0.0/0"
                        }
                    ],
                    "ranges": [
                        [
                            {
                                "subnet": "10.12.0.0/16",
                                "gateway": "10.12.0.1"
                            }
                        ]
                    ]
                }
            },
            {
                "type": "portmap",
                "capabilities": {
                    "portMappings": true
                }
            },
            {
                "type": "firewall",
                "backend": "iptables"
            },
            {
                "type": "tuning"
            }
        ]
    }

    Examples:
    >>> len(cni_podman_bridge_conf["plugins"])
    4
    >>> cni_podman_bridge_conf["plugins"][0]["ipMasq"]
    True
"""

from insights.specs import Specs
from insights import JSONParser, parser


@parser(Specs.cni_podman_bridge_conf)
class CNIPodmanBridgeConf(JSONParser):
    """
    Class for file: /etc/cni/net.d/87-podman-bridge.conflist
    """
    pass



"""
Uptime - command ``/usr/bin/uptime``
====================================

Parse the output of the ``uptime`` command into six attributes:

* ``currtime``: the time on the system as a string.
* ``loadavg``: a three element array of strings for the one, five and
  fifteen minute load averages.
* ``updays``: a string of the number of days the system has been up, or ''
  if the system has been running for less than a day.
* ``uphhmm``: a string of the fraction of a day in hours and minutes that the
  system has been running.  Times reported by ``uptime`` as e.g. '30 mins' are
  converted in to hh:mm format.
* ``users``: a string containing the number of users ``uptime`` reports as
  using the system.
* ``uptime``: a ``datetime.timedelta`` object of the total duration of uptime.

These can also be queried as named keys in the ``data`` attribute.

Sample output::

     11:51:06 up  3:17,  1 user,  load average: 0.12, 0.20, 0.28

Examples:
    >>> uptime = shared[Uptime]
    >>> from datetime import timedelta
    >>> uptime.uptime > timedelta(days=1)
    False
    >>> uptime.updays
    ''
    >>> uptime.users
    '1'
    >>> uptime.loadavg[1]
    '0.20'
    >>> uptime.data['currtime']
    '11:51:06'
"""
import datetime
import re

from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.uptime)
class Uptime(CommandParser):
    """Parser class to parse the output of ``uptime``."""

    def parse_content(self, content):
        self.currtime = None
        self.loadavg = None
        self.updays = None
        self.uphhmm = None
        self.users = None
        self.uptime = None
        uptime_info = {}
        self.data = uptime_info

        # Use only the first line
        line = content[0].strip()

        #  10:55:22 up 40 days, 21:17,  1 user,  load average: 0.49, 0.12, 0.04
        curr_time_re = r'(?P<currtime>\d\d:\d\d:\d\d)'
        # Duration Variations:
        # 30 mins |  5:55 | 1 day, 3 mins | 2 days,  8:05
        duration_re = r' up  ?(?:(?P<days>\d+) days?)?(?:, )?' + \
            r'(?:(?:(?P<mins>\d+) mins?)?|(?: ?(?P<hhmm>\d?\d:\d\d)))'
        users_re = r',  ?(?P<users>\d+) users?'
        load_re = r',  ?load average: (?P<load1>\d+.\d\d), (?P<load5>\d+.\d\d), (?P<load15>\d+.\d\d)'
        line_rex = re.compile(curr_time_re + duration_re + users_re + load_re)

        match = line_rex.search(line)
        if not match:
            raise ParseException("No uptime data found on '{line}'".format(line=line))

        uptime_info['updays'] = match.group('days') if match.group('days') else ''
        uptime_info['currtime'] = match.group('currtime')
        uptime_info['uphhmm'] = match.group('hhmm') if match.group('hhmm') else ''
        uptime_info['users'] = match.group('users')
        uptime_info['loadavg'] = [match.group('load1'), match.group('load5'), match.group('load15')]
        uptime_info['uptime'] = datetime.timedelta()
        if not match.group('hhmm') and match.group('mins'):
            uptime_info['uphhmm'] = '00:{m:02}'.format(m=int(match.group('mins')))

        if uptime_info['uphhmm']:
            hours, mins = uptime_info['uphhmm'].split(':')
            uptime_info['uptime'] += datetime.timedelta(hours=int(hours), minutes=int(mins))
        if uptime_info['updays']:
            uptime_info['uptime'] += datetime.timedelta(days=int(uptime_info['updays']))

        for k, v in uptime_info.items():
            setattr(self, k, v)

        self.data = uptime_info



"""
Custom datasources to get the iris working configuration/log files.
"""
import os

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.parsers.iris import IrisList, IrisCpf


@datasource(IrisList, HostContext)
def iris_working_configuration(broker):
    """
    This datasource get information from ``iris.cpf`` file.

    Returns:
        list: the file path of the ``iris.cpf`` files

    Raises:
        SkipComponent: there is no `iris.cpf` file
    """
    all_paths = []
    for item in broker[IrisList]:
        conf_directory = item['directory']
        conf_file = item['conf file'].split()[0].strip()

        file_path = os.path.join(conf_directory, conf_file)
        if os.path.isfile(file_path):
            all_paths.append(file_path)
    if all_paths:
        return all_paths
    raise SkipComponent


@datasource(IrisCpf, HostContext)
def iris_working_messages_log(broker):
    """
    This datasource get information from ``messages.log`` file.

    Returns:
        list: the file path of the ``messages.log`` files

    Raises:
        SkipComponent: there is no `messages.log` file
    """
    iris_cpf = broker[IrisCpf]
    all_paths = []
    for item in iris_cpf:
        if item.has_option('config', 'console') and item.has_option('Databases', 'IRISSYS'):
            primary_log_path = item.get("config", "console").split(",")[-1]
            if primary_log_path:
                log_path = primary_log_path
            else:
                secondary_log_path = item.get("Databases", "IRISSYS")
                log_path = os.path.join(secondary_log_path, 'messages.log')
            if os.path.isfile(log_path):
                all_paths.append(log_path)
    if all_paths:
        return all_paths
    raise SkipComponent



"""
Custom datasource compliance
============================

Only when option `--compliance` is specified, the insights collection will
collect data for compliance.  When it is specified, only the Insights Spec
:py:attr:`insights.specs.default.DefaultSpecs.compliance` will be
collected. To collect other specs within the collection of compliance data,
add them to the "persist" section in the "compliance_manifest"
pre-defined in the :py:mod:`insights.specs.datasources.manifests`
"""
import logging
import os
import pkgutil
import sys
import yaml

from tempfile import NamedTemporaryFile

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.os_release import OsRelease
from insights.parsers.redhat_release import RedhatRelease
from insights.specs.datasources.compliance import ComplianceClient, SSG_PACKAGE, REQUIRED_PACKAGES

# Since XPath expression is not supported by the ElementTree in Python 2.6,
# import insights.contrib.ElementTree when running python is prior to 2.6 for compatibility.
# Script insights.contrib.ElementTree is the same with xml.etree.ElementTree in Python 2.7.14
# Otherwise, import defusedxml.ElementTree to avoid XML vulnerabilities,
# if dependency not installed import xml.etree.ElementTree instead.
if sys.version_info[0] == 2 and sys.version_info[1] <= 6:
    import insights.contrib.ElementTree as ET
else:
    try:
        import defusedxml.ElementTree as ET
    except:
        import xml.etree.ElementTree as ET


logger = logging.getLogger(__name__)


@datasource(HostContext)
def compliance_enabled(broker):
    insights_config = broker.get('client_config')
    # Run compliance only if `--compliance` options are enabled to insights-client
    if not (insights_config and hasattr(insights_config, 'compliance') and insights_config.compliance):
        raise SkipComponent("Only collect compliance data when specifically requested via --compliance option")
    return True


@datasource(compliance_enabled, [OsRelease, RedhatRelease], HostContext)
def os_version(broker):
    os_release = None
    if OsRelease in broker:
        os_release = broker[OsRelease].get("VERSION_ID")
    if not os_release and RedhatRelease in broker:
        os_release = broker[RedhatRelease].version
    if os_release:
        return os_release.split('.')
    raise SkipComponent('Cannot determine OS Version.')


@datasource(compliance_enabled, InstalledRpms, HostContext)
def package_check(broker):
    rpms = broker[InstalledRpms]
    missed = [rpm for rpm in REQUIRED_PACKAGES if rpm not in rpms]
    if missed:
        msg = 'Missing required packages for compliance scanning. Please ensure the following packages are installed: {0}\n'.format(', '.join(missed))
        logger.error(msg)
        raise SkipComponent(msg)

    return rpms.newest(SSG_PACKAGE).version


# timeout=0 disables the datasource timeout alarm
# allowing compliance to run for as long as necessary
@datasource(os_version, package_check, HostContext, timeout=0)
def compliance(broker):
    """
    Custom datasource to collects content for Insights Compliance
    """
    try:
        insights_config = broker.get('client_config')
        # Manifest for compliance is wrapped in the 'insights_config'
        # and was set in the `insights.client.config.InsightsConfig._imply_options`

        # TODO: add the support of new options here
        # # --compliance-policies was called
        # if config.compliance_policies:
        #     result = ComplianceClient(
        #         os_version=broker[os_version],
        #         ssg_version=broker[package_check],
        #         config=insights_config
        #     ).assignable_policies()
        #     sys.exit(result)

        # # --compliance-assign was called
        # if config.compliance_assign:
        #     result = ComplianceClient(
        #         os_version=broker[os_version],
        #         ssg_version=broker[package_check],
        #         config=insights_config
        #     ).policy_link(insights_config.compliance_assign, 'patch')
        #     sys.exit(result)

        # # --compliance-unassign was called
        # if config.compliance_unassign:
        #     result = ComplianceClient(
        #         os_version=broker[os_version],
        #         ssg_version=broker[package_check],
        #         config=insights_config
        #     ).policy_link(insights_config.compliance_unassign, 'delete')
        #     sys.exit(result)

        compliance = ComplianceClient(
            os_version=broker[os_version],
            ssg_version=broker[package_check],
            config=insights_config)
        # Preparations
        initial_profiles = compliance.get_initial_profiles()
        matching_os_profiles = compliance.get_profiles_matching_os()
        profiles = compliance.profile_union_by_ref_id(matching_os_profiles, initial_profiles)
        results_need_repair = compliance.results_need_repair()
        compliance_result = list()
        # OSCAP scan
        for profile in profiles:
            profile_ref_id = profile['attributes']['ref_id']
            file_name = 'oscap_results-{0}'.format(profile_ref_id)
            results_file = NamedTemporaryFile(prefix=file_name, suffix='.xml', delete=True)
            tailoring_file = compliance.download_tailoring_file(profile)
            compliance.run_scan(
                profile_ref_id,
                compliance.find_scap_policy(profile_ref_id),
                results_file.name,
                tailoring_file_path=tailoring_file)
            # TODO: align with core collection
            if insights_config.obfuscate:
                tree = ET.parse(results_file.name)
                # Retrieve the list of xpaths that need to be obfuscated
                xpaths = yaml.load(pkgutil.get_data('insights', 'compliance_obfuscations.yaml'), Loader=yaml.SafeLoader)
                # Obfuscate IP addresses in the XCCDF report
                compliance.obfuscate(tree, xpaths['obfuscate'])
                if insights_config.obfuscate_hostname:
                    # Obfuscate the hostname in the XCCDF report
                    compliance.obfuscate(tree, xpaths['obfuscate_hostname'])
                # Overwrite the results file with the obfuscations
                tree.write(results_file.name)
            if tailoring_file:
                os.remove(tailoring_file)
            # Store result as DatasourceProvider
            with open(results_file.name, 'r') as f:
                content = [l.rstrip("\n") for l in f]
                if results_need_repair:
                    content = compliance.repair_results(results_file.name, content)
                compliance_result.append(
                    DatasourceProvider(
                        content=content,  # The actual result content here
                        relative_path='{0}.xml'.format(file_name)
                    )
                )
        if compliance_result:
            return compliance_result

    except Exception as err:
        from traceback import format_exc
        err_msg = "Unexpected exception in compliance: {0}".format(str(err))
        logger.error(err_msg)
        logger.debug(format_exc())
        raise SkipComponent(err_msg)
    raise SkipComponent("No scan results were produced")



from insights.tests import context_wrap
from insights.parsers.qemu_conf import QemuConf

qemu_conf_content = """
   vnc_listen = "0.0.0.0"

   vnc_auto_unix_socket = 1
   vnc_tls = 1
   # comment line
   vnc_tls_x509_cert_dir = "/etc/pki/libvirt-vnc"
   security_driver = "selinux"    #inline comment
   cgroup_device_acl = [
    "/dev/null", "/dev/full", "/dev/zero",
    "/dev/random", "/dev/urandom",
    "/dev/ptmx", "/dev/kvm", "/dev/kqemu",
    "/dev/rtc","/dev/hpet", "/dev/vfio/vfio"
    ]
"""

qemu_conf_comment = """
    # comment line
    # comment line
"""


def test_sssd_conf():
    result = QemuConf(context_wrap(qemu_conf_content))
    assert result.get("vnc_listen") == '0.0.0.0'
    assert result.get("vnc_tls") == '1'
    assert "/dev/zero" in result.get('cgroup_device_acl')
    assert result.get('security_driver') == 'selinux'
    assert isinstance(result.get('cgroup_device_acl'), list)

    result = QemuConf(context_wrap(qemu_conf_comment))
    assert result.data == {}



# -*- coding: UTF-8 -*-
import os
try:
    from unittest.mock import patch, Mock
except Exception:
    from mock import patch, Mock

from pytest import raises
from tempfile import NamedTemporaryFile

from insights.core.exceptions import SkipComponent
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.os_release import OsRelease
from insights.parsers.redhat_release import RedhatRelease
from insights.specs.datasources.compliance.compliance_ds import os_version, package_check, compliance
from insights.tests import context_wrap

OS_RELEASE = """
NAME="Red Hat Enterprise Linux"
VERSION="8.10 (Ootpa)"
ID="rhel"
ID_LIKE="fedora"
VERSION_ID="8.10"
PLATFORM_ID="platform:el8"
PRETTY_NAME="Red Hat Enterprise Linux 8.10 (Ootpa)"
ANSI_COLOR="0;31"
CPE_NAME="cpe:/o:redhat:enterprise_linux:8::baseos"
HOME_URL="https://www.redhat.com/"
DOCUMENTATION_URL="https://access.redhat.com/documentation/en-us/red_hat_enterprise_linux/8"
BUG_REPORT_URL="https://bugzilla.redhat.com/"
"""
REDHAT_RELEASE = """
Red Hat Enterprise Linux release 8.10 (Ootpa)
"""
RPMS_JSON_NG = """
{"name": "grub2-tools","version": "2.02","epoch": "1","release": "0.34.el7_2","arch": "x86_64","installtime": "Fri 24 Jun 2016 04:18:01 PM EDT","buildtime": "1450199819","rsaheader": "RSA/SHA256, Wed 23 Dec 2015 04:22:27 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "grub2-2.02-0.34.el7_2.src.rpm"}
{"name": "kbd-legacy","version": "1.15.5","epoch": "(none)","release": "11.el7","arch": "noarch","installtime": "Fri 06 May 2016 03:53:32 PM EDT","buildtime": "1412004323","rsaheader": "RSA/SHA256, Tue 16 Dec 2014 10:02:14 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "kbd-1.15.5-11.el7.src.rpm"}
{"name": "jboss-servlet-3.0-api","version": "1.0.1","epoch": "(none)","release": "9.el7","arch": "noarch","installtime": "Thu 02 Jun 2016 05:10:30 PM EDT","buildtime": "1388211302","rsaheader": "RSA/SHA256, Tue 01 Apr 2014 02:51:30 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "jboss-servlet-3.0-api-1.0.1-9.el7.src.rpm"}
{"name": "bash","version": "4.2.46","epoch": "(none)","release": "19.el7","arch": "x86_64","installtime": "Fri 06 May 2016 03:52:13 PM EDT","buildtime": "1436354006","rsaheader": "RSA/SHA256, Wed 07 Oct 2015 01:14:10 PM EDT, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "bash-4.2.46-19.el7.src.rpm"}
{"name": "ca-certificates","version": "2015.2.6","epoch": "(none)","release": "70.1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
"""
RPMS_JSON = RPMS_JSON_NG + """
{"name": "scap-security-guide","version": "0.1.72","epoch": "(none)","release": "1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
{"name": "openscap-scanner","version": "1.3.10","epoch": "(none)","release": "1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
{"name": "openscap","version": "1.3.10","epoch": "(none)","release": "1.el7_2","arch": "noarch","installtime": "Fri 24 Jun 2016 04:18:04 PM EDT","buildtime": "1453976868","rsaheader": "RSA/SHA256, Tue 02 Feb 2016 09:45:04 AM EST, Key ID 199e2f91fd431d51","dsaheader": "(none)","srpm": "ca-certificates-2015.2.6-70.1.el7_2.src.rpm"}
"""


def test_os_version():
    os_good = OsRelease(context_wrap(OS_RELEASE))
    rh_good = RedhatRelease(context_wrap(REDHAT_RELEASE))
    broker = {OsRelease: os_good, RedhatRelease: rh_good}
    result = os_version(broker)
    assert result == ['8', '10']

    os_ng = OsRelease(context_wrap('test=abc'))
    broker = {OsRelease: os_ng, RedhatRelease: rh_good}
    result = os_version(broker)
    assert result == ['8', '10']

    rh_ng = RedhatRelease(context_wrap('test abc'))
    broker = {OsRelease: os_good, RedhatRelease: rh_ng}
    result = os_version(broker)
    assert result == ['8', '10']

    broker = {OsRelease: os_ng, RedhatRelease: rh_ng}
    with raises(SkipComponent):
        os_version(broker)


def test_package_check():
    rpms = InstalledRpms(context_wrap(RPMS_JSON))
    broker = {InstalledRpms: rpms}
    result = package_check(broker)
    assert result == '0.1.72'

    rpms = InstalledRpms(context_wrap(RPMS_JSON_NG))
    broker = {InstalledRpms: rpms}
    with raises(SkipComponent) as sc:
        package_check(broker)
        msg = str(sc)
        assert "scap-security-guide" in msg
        assert "openscap-scanner" in msg
        assert "openscap" in msg


@patch("insights.core.spec_factory.log")
@patch("insights.specs.datasources.compliance.InsightsConnection")
@patch("insights.specs.datasources.compliance.ComplianceClient.get_initial_profiles",
        return_value=[{'attributes': {'ref_id': 'foo', 'tailored': False}}])
@patch("insights.specs.datasources.compliance.ComplianceClient.get_profiles_matching_os",
        return_value=[])
@patch("insights.specs.datasources.compliance.ComplianceClient.run_scan",
        return_value=None)
@patch("insights.specs.datasources.compliance.ComplianceClient.find_scap_policy",
        return_value='test.xml')
@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None, compressor='gz', obfuscate=False)
def test_compliance_ds(config, _xml, run_scan, pm_os, init_p, conn, log):
    conn.session.get = Mock(return_value=Mock(status_code=200, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    ret = compliance(broker)
    assert len(ret) == 1
    assert ret[0].relative_path == 'oscap_results-foo.xml'
    assert ret[0].content == []
    log.debug.assert_called_once_with('File is empty (after filtering): %s', '/' + ret[0].relative_path)


@patch("insights.specs.datasources.compliance.compliance_ds.NamedTemporaryFile")
@patch("insights.specs.datasources.compliance.InsightsConnection")
@patch("insights.specs.datasources.compliance.ComplianceClient.get_initial_profiles",
        return_value=[{'attributes': {'ref_id': 'foo', 'tailored': False}}])
@patch("insights.specs.datasources.compliance.ComplianceClient.get_profiles_matching_os",
        return_value=[])
@patch("insights.specs.datasources.compliance.ComplianceClient.run_scan",
        return_value=None)
@patch("insights.specs.datasources.compliance.ComplianceClient.find_scap_policy",
        return_value='test.xml')
@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None, compressor='gz', obfuscate=True)
def test_compliance_ds_with_obfuscation(config, _xml, run_scan, pm_os, init_p, conn, ntmpf):
    tmp_file = NamedTemporaryFile(mode='w', delete=False)
    tmp_file.write("""
<xml>
    <TestResult xmlns="http://checklists.nist.gov/xccdf/1.2">
      <target-address>obfuscate</target-address>
      <target-facts>
        <fact name="urn:xccdf:fact:asset:identifier:ipv4" type="string">obfuscate</fact>
        <fact name="urn:xccdf:fact:asset:identifier:ipv6" type="string">obfuscate</fact>
      </target-facts>
    </TestResult>
    <arf xmlns="http://scap.nist.gov/schema/asset-identification/1.1">
      <ip-address>
        <ip-v4>obfuscate</ip-v4>
      </ip-address>
      <ip-address>
        <ip-v6>obfuscate</ip-v6>
      </ip-address>
      <mac-address>obfuscate</mac-address>
    </arf>
    <oval xmlns="http://oval.mitre.org/XMLSchema/oval-system-characteristics-5">
      <system-info>
        <interfaces>
          <interface>
            <ip_address>obfuscate</ip_address>
            <mac_address>obfuscate</mac_address>
          </interface>
        </interfaces>
      </system-info>
    </oval>
</xml>
    """)
    ntmpf.return_value = tmp_file
    tmp_file.close()
    conn.session.get = Mock(return_value=Mock(status_code=200, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    ret = compliance(broker)
    assert ret[0].relative_path == 'oscap_results-foo.xml'

    obfuscated_results = open(str(tmp_file.name)).read()
    os.remove(tmp_file.name)
    assert '<target-address>obfuscate</target-address>' not in obfuscated_results
    assert '<fact name="urn:xccdf:fact:asset:identifier:ipv4" type="string">obfuscate</fact>' not in obfuscated_results
    assert '<fact name="urn:xccdf:fact:asset:identifier:ipv6" type="string">obfuscate</fact>' not in obfuscated_results
    assert '<ip-v4>obfuscate</ip-v4>' not in obfuscated_results
    assert '<ip-v6>obfuscate</ip-v6>' not in obfuscated_results
    assert '<mac-address>obfuscate</mac-address>' not in obfuscated_results
    assert '<ip_address>obfuscate</ip_address>' not in obfuscated_results
    assert '<mac_address>obfuscate</mac_address>' not in obfuscated_results

    content = ' '.join(ret[0].content)
    assert '<target-address>obfuscate</target-address>' not in content
    assert '<fact name="urn:xccdf:fact:asset:identifier:ipv4" type="string">obfuscate</fact>' not in content
    assert '<fact name="urn:xccdf:fact:asset:identifier:ipv6" type="string">obfuscate</fact>' not in content
    assert '<ip-v4>obfuscate</ip-v4>' not in content
    assert '<ip-v6>obfuscate</ip-v6>' not in content
    assert '<mac-address>obfuscate</mac-address>' not in content
    assert '<ip_address>obfuscate</ip_address>' not in content
    assert '<mac_address>obfuscate</mac_address>' not in content


@patch("insights.specs.datasources.compliance.compliance_ds.NamedTemporaryFile")
@patch("insights.specs.datasources.compliance.InsightsConnection")
@patch("insights.specs.datasources.compliance.ComplianceClient.get_initial_profiles",
        return_value=[{'attributes': {'ref_id': 'foo', 'tailored': False}}])
@patch("insights.specs.datasources.compliance.ComplianceClient.get_profiles_matching_os",
        return_value=[])
@patch("insights.specs.datasources.compliance.ComplianceClient.run_scan",
        return_value=None)
@patch("insights.specs.datasources.compliance.ComplianceClient.find_scap_policy",
        return_value='test.xml')
@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None, compressor='gz', obfuscate=True, obfuscate_hostname=True)
def test_compliance_ds_with_hostname_obfuscation(config, _xml, run_scan, pm_os, init_p, conn, ntmpf):
    tmp_file = NamedTemporaryFile(mode='w', delete=False)
    tmp_file.write("""
<xml>
    <TestResult xmlns="http://checklists.nist.gov/xccdf/1.2">
      <target>obfuscate</target>
      <target-address>obfuscate</target-address>
      <target-facts>
        <fact name="urn:xccdf:fact:asset:identifier:fqdn" type="string">obfuscate</fact>
        <fact name="urn:xccdf:fact:asset:identifier:host_name" type="string">obfuscate</fact>
      </target-facts>
    </TestResult>
    <arf xmlns="http://scap.nist.gov/schema/asset-identification/1.1">
      <ip-address>
        <ip-v4>obfuscate</ip-v4>
      </ip-address>
      <ip-address>
        <ip-v6>obfuscate</ip-v6>
      </ip-address>
      <mac-address>obfuscate</mac-address>
    </arf>
    <oval xmlns="http://oval.mitre.org/XMLSchema/oval-system-characteristics-5">
      <system_info>
        <interfaces>
          <interface>
            <ip_address>obfuscate</ip_address>
            <mac_address>obfuscate</mac_address>
          </interface>
        </interfaces>
        <primary_host_name>obfuscate</primary_host_name>
        <node_name>obfuscate</node_name>
      </system_info>
    </oval>
    <ai xmlns="http://scap.nist.gov/schema/asset-identification/1.1">
      <hostname>obfuscate</hostname>
    </ai>
</xml>
    """)
    ntmpf.return_value = tmp_file
    tmp_file.close()
    conn.session.get = Mock(return_value=Mock(status_code=200, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    broker = {os_version: ['8', '10'], package_check: '0.1.73', 'client_config': config}
    ret = compliance(broker)

    obfuscated_results = open(str(tmp_file.name)).read()
    os.remove(tmp_file.name)
    assert '<target-address>obfuscate</target-address>' not in obfuscated_results
    assert '<fact name="urn:xccdf:fact:asset:identifier:fqdn" type="string">obfuscate</fact>' not in obfuscated_results
    assert '<fact name="urn:xccdf:fact:asset:identifier:host_name" type="string">obfuscate</fact>' not in obfuscated_results
    assert '<ip-v4>obfuscate</ip-v4>' not in obfuscated_results
    assert '<ip-v6>obfuscate</ip-v6>' not in obfuscated_results
    assert '<mac-address>obfuscate</mac-address>' not in obfuscated_results
    assert '<ip_address>obfuscate</ip_address>' not in obfuscated_results
    assert '<mac_address>obfuscate</mac_address>' not in obfuscated_results
    assert '<fqdn>obfuscate</fqdn>' not in obfuscated_results
    assert '<hostname>obfuscate</hostname>' not in obfuscated_results
    assert '<target>obfuscate</target>' not in obfuscated_results
    assert '<primary_host_name>obfuscate</primary_host_name>' not in obfuscated_results
    assert '<node_name>obfuscate</node_name>' not in obfuscated_results

    content = ' '.join(ret[0].content)
    assert '<target-address>obfuscate</target-address>' not in content
    assert '<fact name="urn:xccdf:fact:asset:identifier:fqdn" type="string">obfuscate</fact>' not in content
    assert '<fact name="urn:xccdf:fact:asset:identifier:host_name" type="string">obfuscate</fact>' not in content
    assert '<ip-v4>obfuscate</ip-v4>' not in content
    assert '<ip-v6>obfuscate</ip-v6>' not in content
    assert '<mac-address>obfuscate</mac-address>' not in content
    assert '<ip_address>obfuscate</ip_address>' not in content
    assert '<mac_address>obfuscate</mac_address>' not in content
    assert '<fqdn>obfuscate</fqdn>' not in content
    assert '<hostname>obfuscate</hostname>' not in content
    assert '<target>obfuscate</target>' not in content
    assert '<primary_host_name>obfuscate</primary_host_name>' not in content
    assert '<node_name>obfuscate</node_name>' not in content


@patch("insights.specs.datasources.compliance.compliance_ds.NamedTemporaryFile")
@patch("insights.specs.datasources.compliance.InsightsConnection")
@patch("insights.specs.datasources.compliance.ComplianceClient.get_initial_profiles",
        return_value=[{'attributes': {'ref_id': 'foo', 'tailored': False}}])
@patch("insights.specs.datasources.compliance.ComplianceClient.get_profiles_matching_os",
        return_value=[])
@patch("insights.specs.datasources.compliance.ComplianceClient.run_scan",
        return_value=None)
@patch("insights.specs.datasources.compliance.ComplianceClient.find_scap_policy",
        return_value='test.xml')
@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None, compressor='gz')
def test_compliance_ds_with_results_repaired(config, _xml, run_scan, pm_os, init_p, conn, ntmpf):
    tmp_file = NamedTemporaryFile(mode='w', delete=False)
    tmp_file.write("""
<xml>
  <version>0.9</version>
</xml>
    """)
    ntmpf.return_value = tmp_file
    tmp_file.close()
    conn.session.get = Mock(return_value=Mock(status_code=200, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    broker = {os_version: ['8', '10'], package_check: '0.1.25', 'client_config': config}
    ret = compliance(broker)
    os.remove(tmp_file.name)

    # repaired data is not write back to the file
    content = ' '.join(ret[0].content)
    assert '<version>0.1.25</version>' in content



import pytest

from insights.core.exceptions import ParseException
from insights.parsers.satellite_version import Satellite6Version
from insights.tests import context_wrap

satellite_version = """
COMMAND> cat /usr/share/foreman/lib/satellite/version.rb

module Satellite
  VERSION = "6.1.3"
end
"""

no_sat = """
scdb-1.15.8-1.el6sat.noarch                                 Wed May 18 14:48:14 2016
scl-utils-20120927-27.el6_6.x86_64                          Wed May 18 14:18:16 2016
SDL-1.2.14-6.el6.x86_64                                     Wed May 18 14:16:25 2016
"""


def test_get_sat6_version():
    result = Satellite6Version(context_wrap(satellite_version, path='satellite_version'))
    assert result.full == "6.1.3"
    assert result.version == "6.1.3"
    assert result.major == 6
    assert result.minor == 1
    assert result.release is None


def test_get_no_sat_version():
    with pytest.raises(ParseException) as e:
        Satellite6Version(context_wrap(no_sat, path='satellite_version'))
    assert "Cannot parse satellite version" in str(e)



import mock
import pytest

from mock.mock import Mock

from insights.core import filters
from insights.core.exceptions import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.rpm_pkgs import LocalSpecs, pkgs_with_writable_dirs, rpm_v_pkg_list
from insights.specs import Specs

RPM_CMD = """
httpd-core; httpd-core-2.4.53-7.el9.x86_64; /usr/share/doc/httpd-core; drwxr-xr-x; apache; root; Red Hat, Inc.
httpd-core; httpd-core-2.4.53-7.el9.x86_64; /usr/share/doc/httpd-core/CHANGES; -rw-r--r--; root; root; Red Hat, Inc.
postgresql-server; postgresql-server-13.7-1.el9_0.x86_64; /var/lib/pgsql; drwx------; postgres; postgres; Red Hat, Inc.
polkit; polkit-0.117-10.el9_0.x86_64; /usr/lib/polkit-1; drwxrwxr-x; polkitd; polkitd; Red Hat, Inc.
polkit; polkit-0.117-10.el9_0.x86_64; /usr/lib/polkit-1/polkitd; -rwxr-xr-x; root; root; Red Hat, Inc.
""".strip()

RPM_EXPECTED = ["httpd-core|httpd-core-2.4.53-7.el9.x86_64|Red Hat, Inc."]

RPM_BAD_CMD = "bash: rpm: command not found..."

RPM_EMPTY_CMD = ""

RELATIVE_PATH = "insights_commands/rpm_pkgs"


def get_users():
    return ["apache", "postgres"]


def get_groups(users):
    return ["apache", "postgres"]


@mock.patch("insights.specs.datasources.rpm_pkgs.get_users", get_users)
@mock.patch("insights.specs.datasources.rpm_pkgs.get_groups", get_groups)
def test_rpm():
    rpm_args = Mock()
    rpm_args.content = RPM_CMD.splitlines()
    broker = {LocalSpecs.rpm_args: rpm_args}

    result = pkgs_with_writable_dirs(broker)
    expected = DatasourceProvider(content=RPM_EXPECTED, relative_path=RELATIVE_PATH)
    assert result
    assert isinstance(result, DatasourceProvider)
    assert sorted(result.content) == sorted(expected.content)
    assert result.relative_path == expected.relative_path


@pytest.mark.parametrize("no_rpm", [RPM_BAD_CMD, RPM_EMPTY_CMD])
def test_no_rpm(no_rpm):
    rpm_args = Mock()
    rpm_args.content = no_rpm.splitlines()
    broker = {LocalSpecs.rpm_args: rpm_args}

    with pytest.raises(SkipComponent):
        pkgs_with_writable_dirs(broker)


def setup_function(func):
    if func is test_pkgs_list_empty:
        pass
    if func is test_rpm_v_pkgs:
        filters.add_filter(Specs.rpm_V_package_list, ['coreutils', 'procps', 'procps-ng', 'shadow-utils', 'passwd', 'sudo', 'chrony', 'findutils', 'glibc'])


def teardown_function():
    if Specs.rpm_V_package_list in filters._CACHE:
        del filters._CACHE[Specs.rpm_V_package_list]


def test_pkgs_list_empty():
    with pytest.raises(SkipComponent):
        rpm_v_pkg_list({})


def test_rpm_v_pkgs():
    ret = rpm_v_pkg_list({})
    assert ret == ['chrony', 'coreutils', 'findutils', 'glibc', 'passwd', 'procps', 'procps-ng', 'shadow-utils', 'sudo']



from insights import dr, parser
from insights.core.plugins import component


@component()
def data():
    return [1, 2, 3, 4, 5]


@parser(data)
class Lax(object):
    count = 0

    def __init__(self, d):
        if Lax.count > 3:
            raise Exception("Lax")
        Lax.count += 1


@parser(data, continue_on_error=False)
class Boom(object):
    count = 0

    def __init__(self, d):
        if Boom.count > 3:
            raise Exception("Boom")
        Boom.count += 1


def test_dont_continue_on_error():
    Boom.count = 0
    broker = dr.run(Boom)
    assert Boom not in broker


def test_continue_on_error():
    Lax.count = 0
    broker = dr.run(Lax)
    assert len(broker[Lax]) == 4



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import nvme_core_io_timeout
from insights.parsers.nvme_core_io_timeout import NVMeCoreIOTimeout
from insights.tests import context_wrap

NVME_CORE_IO_TIMEOUT = "4294967295"

NVME_CORE_IO_TIMEOUT_INVALID_1 = """
30
255
""".strip()

NVME_CORE_IO_TIMEOUT_INVALID_2 = "FF"


def test_nvme_core_io_timeout_se():
    with pytest.raises(SkipComponent):
        NVMeCoreIOTimeout(context_wrap(''))

    with pytest.raises(SkipComponent):
        NVMeCoreIOTimeout(context_wrap(NVME_CORE_IO_TIMEOUT_INVALID_1))


def test_nvme_core_io_timeout_pe():
    with pytest.raises(ParseException) as pe:
        NVMeCoreIOTimeout(context_wrap(NVME_CORE_IO_TIMEOUT_INVALID_2))
    assert 'Unexpected content' in str(pe)


def test_nvme_core_io_timeout():
    nciotmo = NVMeCoreIOTimeout(context_wrap(NVME_CORE_IO_TIMEOUT))
    assert nciotmo.val == 4294967295


def test_doc_examples():
    env = {
            'nciotmo': NVMeCoreIOTimeout(context_wrap(NVME_CORE_IO_TIMEOUT))
          }
    failed, total = doctest.testmod(nvme_core_io_timeout, globs=env)
    assert failed == 0



"""
``test dumpe2fs_h``
===================
"""
from insights.parsers import dumpe2fs_h
from insights.tests import context_wrap

DUMPE2FS = """
dumpe2fs 1.41.12 (17-May-2010)
Filesystem volume name:   <none>
Last mounted on:          /usr
Filesystem UUID:          1b332c5d-2410-4934-9118-466f8a14841f
Filesystem magic number:  0xEF53
Filesystem revision #:    1 (dynamic)
Filesystem features:      has_journal ext_attr resize_inode dir_index filetype needs_recovery extent flex_bg sparse_super large_file huge_file uninit_bg dir_nlink extra_isize
Filesystem flags:         signed_directory_hash
Default mount options:    user_xattr acl journal_data_writeback

""".strip()

PATH = "sos_commands/filesys/dumpe2fs_-h_.dev.mapper.vg_spcdrhellb01-lv_usr"


def test_dumpe2fs():
    dumpe2fs_obj = dumpe2fs_h.DumpE2fs(context_wrap(DUMPE2FS, path=PATH))
    assert dumpe2fs_obj.dev_name == '/dev/mapper/vg_spcdrhellb01-lv_usr'
    assert dumpe2fs_obj is not None
    assert type(dumpe2fs_obj['Filesystem features']) is list
    assert set(dumpe2fs_obj.get("Filesystem features")) == set([
        'has_journal', 'ext_attr', 'resize_inode', 'dir_index',
        'filetype', 'needs_recovery', 'extent', 'flex_bg', 'sparse_super',
        'large_file', 'huge_file', 'uninit_bg', 'dir_nlink', 'extra_isize'
    ])
    assert type(dumpe2fs_obj.get("Default mount options")) == list
    assert set(dumpe2fs_obj.get("Default mount options")) == \
        set(['user_xattr', 'acl', 'journal_data_writeback'])
    assert dumpe2fs_obj.get('Filesystem magic number') == '0xEF53'
    assert type(dumpe2fs_obj.get("Filesystem flags")) == list
    assert set(dumpe2fs_obj.get('Filesystem flags')) == set(['signed_directory_hash'])
    assert dumpe2fs_obj.get('Filesystem revision #') == '1 (dynamic)'
    assert dumpe2fs_obj.get('Last mounted on') == '/usr'
    assert dumpe2fs_obj.get('Filesystem UUID') == '1b332c5d-2410-4934-9118-466f8a14841f'
    assert dumpe2fs_obj.get('Filesystem volume name') == '<none>'



"""
cpu_online - File ``/sys/devices/system/cpu/online``
====================================================

This parser reads the content of ``/sys/devices/system/cpu/online``.
This file shows the number of online cpu. The format of the content
is string including comma.
"""

from insights.core import ContainerParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.container_cpu_online)
class ContainerCpuOnline(ContainerParser):
    """
    Class ``ContainerCpuOnline`` parses the content of the ``/sys/devices/system/cpu/online`` from containers.

    Attributes:
        cpu_online_set (list): It is used to show the list of online cpu.

        cpu_online_number (int): It is used to display the number of online cpu.

    A small sample of the content of this file looks like::

        0,2-4,7

    Examples:
        >>> type(container_cpu_online_info)
        <class 'insights.parsers.cpu_online.ContainerCpuOnline'>
        >>> container_cpu_online_info.container_id
        '2869b4e2541c'
        >>> container_cpu_online_info.image
        'registry.access.redhat.com/ubi8/nginx-120'
        >>> container_cpu_online_info.cpu_online_set
        ['0', '2', '3', '4', '7']
        >>> container_cpu_online_info.cpu_online_number
        5
    """
    def parse_content(self, content):
        self.cpu_online_set = []
        self.cpu_online_number = 0
        values = content[0].strip().split(",")
        for value in values:
            if "-" in value:
                # Parse the value like "2-4"
                start, end = value.split("-")
                self.cpu_online_set.extend([str(i) for i in range(int(start), int(end) + 1)])
            else:
                self.cpu_online_set.append(value)
        self.cpu_online_number = len(self.cpu_online_set)



"""
LsVarOptMssqlLog - command ``ls -la /var/opt/mssql/log``
========================================================

This parser reads the ``/var/opt/mssql/log`` directory listings and uses the
FileListing parser class to provide a common access to them.

"""

from insights import FileListing, parser, CommandParser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ls_var_opt_mssql_log)
class LsVarOptMssqlLog(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSla` instead.

    A parser for accessing "ls -la /var/opt/mssql/log".

    Examples:
        >>> '/var/opt/mssql/log' in ls_mssql_log
        True
        >>> ls_mssql_log.dir_contains('/var/opt/mssql/log', 'messages')
        False
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsVarOptMssqlLog, "Please use the :class:`insights.parsers.ls.LSla` instead.", "3.5.0")
        super(LsVarOptMssqlLog, self).__init__(*args, **kwargs)



"""
Combiners for getting the earliest expiry date from a lot of SSL certificates
=============================================================================

This module contains the following combiners:

EarliestNginxSSLCertExpireDate - The earliest expire date in a lot of nginx ssl certificates
--------------------------------------------------------------------------------------------
Combiner to get the earliest expire date in a lot of nginx ssl certificates.

EarliestHttpdSSLCertExpireDate - The earliest expire date in a lot of httpd ssl certificates
--------------------------------------------------------------------------------------------
Combiner to get the earliest expire date in a lot of httpd ssl certificates.

EarliestHttpdCertInNSSExpireDate - The earliest expire date in a lot of httpd certificates stored in nss database
-----------------------------------------------------------------------------------------------------------------
Combiner to get the earliest expire date in a lot of httpd certificates stored in nss database.
"""
from insights.core.exceptions import SkipComponent
from insights.core.plugins import combiner
from insights.parsers.certificates_enddate import CertificatesEnddate
from insights.parsers.ssl_certificate import HttpdCertInfoInNSS, NginxSSLCertExpireDate, HttpdSSLCertExpireDate


class EarliestSSLCertExpireDate(object):
    """
    The base class to get the earliest expiry date from a lot of :class:`insights.parsers.ssl_certificate.CertificateInfo` instances.

    Attributes:
        earliest_expire_date (str): The earliest expire date in string format.
        ssl_cert_path (str): The SSL certificate path which is expired first.

    Examples:
        >>> type(ssl_certs)
        <class 'insights.combiners.ssl_certificate.EarliestSSLCertExpireDate'>
        >>> ssl_certs.earliest_expire_date.str
        'Dec 18 07:02:43 2021'
        >>> ssl_certs.ssl_cert_path
        '/test/b.pem'
    """
    def __init__(self, certificate_info_list):
        self.earliest_expire_date = None
        self.ssl_cert_path = None
        for ssl_cert_expiry_date in certificate_info_list:
            if (self.earliest_expire_date is None or
                (isinstance(ssl_cert_expiry_date.get('notAfter', ''), CertificatesEnddate.ExpirationDate) and
                    ssl_cert_expiry_date['notAfter'].datetime < self.earliest_expire_date.datetime)):
                self.earliest_expire_date = ssl_cert_expiry_date['notAfter']
                self.ssl_cert_path = ssl_cert_expiry_date.cert_path
        if self.earliest_expire_date is None:
            raise SkipComponent


@combiner(NginxSSLCertExpireDate)
class EarliestNginxSSLCertExpireDate(EarliestSSLCertExpireDate):
    """
    Combiner to get the earliest expire date in a lot of nginx ssl certificates.

    Examples:
        >>> type(nginx_certs)
        <class 'insights.combiners.ssl_certificate.EarliestNginxSSLCertExpireDate'>
        >>> nginx_certs.earliest_expire_date.str
        'Dec 18 07:02:43 2021'
        >>> nginx_certs.ssl_cert_path
        '/test/d.pem'
    """
    pass


@combiner(HttpdSSLCertExpireDate)
class EarliestHttpdSSLCertExpireDate(EarliestSSLCertExpireDate):
    """
    Combiner to get the earliest expire date in a lot of httpd ssl certificates.

    Examples:
        >>> type(httpd_certs)
        <class 'insights.combiners.ssl_certificate.EarliestHttpdSSLCertExpireDate'>
        >>> httpd_certs.earliest_expire_date.str
        'Dec 18 07:02:43 2021'
        >>> httpd_certs.ssl_cert_path
        '/test/d.pem'
    """
    pass


@combiner(HttpdCertInfoInNSS)
class EarliestHttpdCertInNSSExpireDate(EarliestSSLCertExpireDate):
    """
    Combiner to get the earliest expire date in a lot of httpd certificates stored in NSS database.

    Examples:
        >>> type(httpd_certs_in_nss)
        <class 'insights.combiners.ssl_certificate.EarliestHttpdCertInNSSExpireDate'>
        >>> httpd_certs_in_nss.earliest_expire_date.str
        'Sun Jan 07 05:26:10 2022'
        >>> httpd_certs_in_nss.ssl_cert_path
        ('/etc/httpd/nss', 'testcerta')
    """
    pass



import datetime
import pytest

from mock.mock import patch

from insights.client.config import InsightsConfig
from insights.combiners.ps import Ps
from insights.combiners.services import Services
from insights.core import dr
from insights.core.exceptions import SkipComponent, ContentException
from insights.parsers.hostname import HostnameDefault, Hostname
from insights.parsers.ps import PsAuxcww
from insights.parsers.ros_config import RosConfig
from insights.parsers.systemd.unitfiles import UnitFiles
from insights.specs.datasources import pcp
from insights.specs.datasources.pcp import (
        pcp_enabled,
        pcp_raw_files,
        pmlog_summary_args_pcp_zeroconf,
        pmlog_summary_args,
        ros_collect)
from insights.tests import context_wrap


yesterday = (datetime.date.today() - datetime.timedelta(days=1)).strftime("%Y%m%d")

PS_AUXCWW = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 init
root      1821  0.0  0.0      0     0 ?        S    May31   0:29 kondemand/0
root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 dhclient
pcp      71277  0.0  0.1 127060  8384 ?        S    Oct09   0:06 pmlogger
""".strip()

PS_AUXCWW_NG = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 init
root      1821  0.0  0.0      0     0 ?        S    May31   0:29 kondemand/0
root     20357  0.0  0.0   9120   832 ?        Ss   10:09   0:00 dhclient
""".strip()

LIST_UNIT_FILES = """
UNIT FILE                                   STATE
pmlogger.service                            enabled
pmlogger_check.service                      disabled
pmlogger_daily-poll.service                 static
pmlogger_daily.service                      static
pmproxy.service                             enabled

5 unit files listed.
""".strip()

LIST_UNIT_FILES_no_pmproxy = """
UNIT FILE                                   STATE
pmlogger.service                            enabled
pmlogger_check.service                      disabled
pmlogger_daily-poll.service                 static
pmlogger_daily.service                      static

5 unit files listed.
""".strip()

ROS_CONFIG = """
log mandatory on default {
    mem.util.used
    kernel.all.cpu.user
    disk.all.total
    mem.util.free
}
[access]
disallow .* : all;
disallow :* : all;
allow local:* : enquire;
""".strip()

ROS_CONFIG_NG = """
log mandatory off {
    mem.util.used
    kernel.all.cpu.user
    disk.all.total
    mem.util.free
}
[access]
disallow .* : all;
disallow :* : all;
allow local:* : enquire;
"""

PCP_RAW_FILES = [
    "/var/log/pcp/pmlogger/test/{0}.0.xz".format(yesterday),
    "/var/log/pcp/pmlogger/test/{0}.1.xz".format(yesterday),
    "/var/log/pcp/pmlogger/test/{0}.index".format(yesterday),
    "/var/log/pcp/pmlogger/test/{0}.meta.xz".format(yesterday),
]


def test_pcp_enabled():
    unitfiles = UnitFiles(context_wrap(LIST_UNIT_FILES))
    services = Services(None, unitfiles)
    broker = dr.Broker()
    broker[Services] = services

    result = pcp_enabled(broker)
    assert result is True

    unitfiles = UnitFiles(context_wrap(LIST_UNIT_FILES_no_pmproxy))
    services = Services(None, unitfiles)
    broker = dr.Broker()
    broker[Services] = services

    with pytest.raises(SkipComponent):
        pcp_enabled(broker)


@patch("insights.specs.datasources.pcp.os.path.exists", return_value=True)
@patch("insights.specs.datasources.pcp.os.path.isfile", return_value=True)
def test_pmlog_summary_args(isfile, exists):
    # Case 1: OK
    ros = RosConfig(context_wrap(ROS_CONFIG))
    ps_auxcww = PsAuxcww(context_wrap(PS_AUXCWW))
    ps = Ps(None, None, None, None, ps_auxcww, None)

    broker = dr.Broker()
    broker[Ps] = ps
    broker[RosConfig] = ros

    pcp_log_date = (datetime.date.today() - datetime.timedelta(days=1)).strftime("%Y%m%d")
    mock_file = "/var/log/pcp/pmlogger/ros/%s.index" % (pcp_log_date)

    result = pmlog_summary_args(broker)

    metrics = ' '.join(sorted([i.strip() for i in ROS_CONFIG.split('\n')[1:5]]))
    expected = '{0} {1}'.format(mock_file, metrics)
    assert result == expected

    # Case 2 NG metrics
    ros = RosConfig(context_wrap(ROS_CONFIG_NG))
    broker = dr.Broker()
    broker[Ps] = ps
    broker[RosConfig] = ros

    with pytest.raises(SkipComponent):
        pmlog_summary_args(broker)

    # Case 3 No pmloger proc in ps
    ros = RosConfig(context_wrap(ROS_CONFIG))
    ps_auxcww = PsAuxcww(context_wrap(PS_AUXCWW_NG))
    ps = Ps(None, None, None, None, ps_auxcww, None)
    broker = dr.Broker()
    broker[Ps] = ps
    broker[RosConfig] = ros

    with pytest.raises(SkipComponent):
        pmlog_summary_args(broker)


@patch("insights.specs.datasources.pcp.os.path.exists", return_value=False)
def test_pmlog_summary_args_no_pmloger_file(isfile):
    ros = RosConfig(context_wrap(ROS_CONFIG))
    ps_auxcww = PsAuxcww(context_wrap(PS_AUXCWW))
    ps = Ps(None, None, None, None, ps_auxcww, None)

    broker = dr.Broker()
    broker[Ps] = ps
    broker[RosConfig] = ros

    with pytest.raises(SkipComponent):
        pmlog_summary_args(broker)


def test_ros_collect():
    ic = InsightsConfig(ros_collect=True)
    result = ros_collect({'client_config': ic})
    assert result is True

    ic = InsightsConfig(ros_collect=False)
    with pytest.raises(SkipComponent):
        ros_collect({'client_config': ic})


@patch("insights.specs.datasources.pcp.os.path.getmtime", return_value=1)
@patch("insights.specs.datasources.pcp.glob.glob", return_value=PCP_RAW_FILES)
@patch("insights.specs.datasources.pcp.os.path.isfile", return_value=True)
@patch("insights.specs.datasources.pcp.os.path.exists", return_value=True)
def test_pcp_raw_files(_exists, _isfile, _glob, mtime):
    broker = dr.Broker()
    broker[HostnameDefault] = HostnameDefault(context_wrap("test"))
    broker['insights_config'] = InsightsConfig(ros_collect=True)

    ret = pcp_raw_files(broker)
    # test.rh.com is expected, with mtime=2
    assert sorted(set(ret)) == sorted([
        "/var/log/pcp/pmlogger/test/{0}.0.xz".format(yesterday),
        "/var/log/pcp/pmlogger/test/{0}.1.xz".format(yesterday),
        "/var/log/pcp/pmlogger/test/{0}.index".format(yesterday),
        "/var/log/pcp/pmlogger/test/{0}.meta.xz".format(yesterday)])


@patch("insights.specs.datasources.pcp.os.path.getmtime", return_value=1)
@patch("insights.specs.datasources.pcp.glob.glob", return_value=PCP_RAW_FILES[:2])
@patch("insights.specs.datasources.pcp.os.path.isfile", return_value=True)
@patch("insights.specs.datasources.pcp.os.path.exists", return_value=True)
def test_pcp_raw_files_ab(_exists, _isfile, _glob, mtime):
    broker = dr.Broker()
    broker[HostnameDefault] = HostnameDefault(context_wrap("test"))
    broker[Hostname] = None
    broker['insights_config'] = InsightsConfig(ros_collect=True)

    with pytest.raises(ContentException):
        pcp_raw_files(broker)


@patch("insights.specs.datasources.pcp.os.path.exists", return_value=False)
def test_pcp_raw_files_ab_dir(_exists):
    broker = dr.Broker()
    broker[HostnameDefault] = HostnameDefault(context_wrap("test.rh.com"))
    broker[Hostname] = Hostname(context_wrap("test.rh.com"))
    broker['insights_config'] = InsightsConfig(ros_collect=True)

    with pytest.raises(ContentException):
        pcp_raw_files(broker)


@patch("insights.specs.datasources.pcp.os.path.getmtime", return_value=1)
@patch("insights.specs.datasources.pcp.glob.glob", return_value=PCP_RAW_FILES)
@patch("insights.specs.datasources.pcp.os.path.isfile", return_value=True)
@patch("insights.specs.datasources.pcp.os.path.exists", return_value=True)
def test_pmlog_summary_args_pcp_zeroconf(_exists, _isfile, _glob, mtime):
    broker = dr.Broker()
    broker[HostnameDefault] = HostnameDefault(context_wrap("test"))
    broker['insights_config'] = InsightsConfig(ros_collect=True)

    pm_files = pcp_raw_files(broker)
    pm_index = "/var/log/pcp/pmlogger/test/{0}.index".format(yesterday)
    assert pm_index in pm_files

    broker[pcp_raw_files] = pm_files

    result = pmlog_summary_args_pcp_zeroconf(broker)
    expected = '{0} {1}'.format(pm_index, ' '.join(pcp.pcp_metrics))
    assert result == expected



"""
SshConfig - file for ``ssh client config``
==========================================

This module contains parsers that check the ssh client config files.

Parsers provided by this module are:

EtcSshConfig - file ``/etc/ssh/ssh_config``
-------------------------------------------

EtcSshConfigD - files ``/etc/ssh/ssh_config.d/*``
-------------------------------------------------

ForemanSshConfig - file ``/usr/share/foreman/.ssh/ssh_config``
--------------------------------------------------------------

ForemanProxySshConfig - file ``/usr/share/foreman-proxy/.ssh/ssh_config``
-------------------------------------------------------------------------

"""
from collections import namedtuple

from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


class SshClientConfig(Parser):
    """
    Base class for ssh client configuration file.

    Sample output::

        # This is the ssh client system-wide configuration file.  See
        # ssh_config(5) for more information.  This file provides defaults for
        # users, and the values can be changed in per-user configuration files
        # or on the command line.

        ProxyCommand ssh -q -W %h:%p gateway.example.com

        Host *
                GSSAPIAuthentication yes
        # If this option is set to yes then remote X11 clients will have full access
        # to the original X11 display. As virtually no X11 client supports the untrusted
        # mode correctly we set this to yes.
                ForwardX11Trusted yes
        # Send locale-related environment variables
                SendEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
                SendEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
                SendEnv LC_IDENTIFICATION LC_ALL LANGUAGE
                SendEnv XMODIFIERS

        Host proxytest
            HostName 192.168.122.2

    Attributes:

        global_lines (list): The list of site-wide configuration, as
            namedtuple('KeyValue', ['keyword', 'value', 'line']).
        host_lines (dict): The dict of all host-specific definitions, as
            {'Host_name': [namedtuple('KeyValue', ['keyword', 'value', 'line'])]}

    Examples:
        >>> len(sshconfig.global_lines)
        1
        >>> sshconfig.global_lines[0].keyword
        'ProxyCommand'
        >>> sshconfig.global_lines[0].value
        'ssh -q -W %h:%p gateway.example.com'
        >>> 'Host_*' in sshconfig.host_lines
        True
        >>> sshconfig.host_lines['Host_proxytest'][0].keyword
        'HostName'
        >>> sshconfig.host_lines['Host_proxytest'][0].value
        '192.168.122.2'

    Raises:
        SkipComponent: When input content is empty. Not found any parse results.
    """

    KeyValue = namedtuple('KeyValue', ['keyword', 'value', 'line'])

    def parse_content(self, content):
        self.global_lines = []
        self.host_lines = {}

        _content = get_active_lines(content)
        if not _content:
            raise SkipComponent("Empty content.")

        index_list = [i for i, l in enumerate(_content) if l.startswith('Host ')]
        index = index_list[0] if index_list else len(_content)

        for line in _content[:index]:
            line_splits = [s.strip() for s in line.split(None, 1)]
            kw, val = line_splits[0], line_splits[1] if len(line_splits) == 2 else ''
            self.global_lines.append(self.KeyValue(kw, val, line))

        hostbit = ''
        for line in _content[index:]:
            line_splits = [s.strip() for s in line.split(None, 1)]
            kw, val = line_splits[0], line_splits[1] if len(line_splits) == 2 else ''
            if kw == 'Host':
                hostbit = kw + '_' + val
                self.host_lines[hostbit] = []
            else:
                self.host_lines[hostbit].append(self.KeyValue(kw, val, line))

        if not (self.host_lines or self.global_lines):
            raise SkipComponent("Nothing parsed.")


@parser(Specs.ssh_config)
class EtcSshConfig(SshClientConfig):
    """
    This parser reads the file ``/etc/ssh/ssh_config``

    Sample output::

        #   ProxyCommand ssh -q -W %h:%p gateway.example.com
        #   RekeyLimit 1G 1h
        #
        # Uncomment this if you want to use .local domain
        # Host *.local
        #   CheckHostIP no
        ProxyCommand ssh -q -W %h:%p gateway.example.com

        Host *
            GSSAPIAuthentication yes
        # If this option is set to yes then remote X11 clients will have full access
        # to the original X11 display. As virtually no X11 client supports the untrusted
        # mode correctly we set this to yes.
            ForwardX11Trusted yes
        # Send locale-related environment variables
            SendEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
            SendEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
            SendEnv LC_IDENTIFICATION LC_ALL LANGUAGE
            SendEnv XMODIFIERS

        Host proxytest
            HostName 192.168.122.2

    Attributes:

        global_lines (list): The list of site-wide configuration, as
            namedtuple('KeyValue', ['keyword', 'value', 'line']).
        host_lines (dict): The dict of all host-specific definitions, as
            {'Host_name': [namedtuple('KeyValue', ['keyword', 'value', 'line'])]}

    Examples:
        >>> len(etcsshconfig.global_lines)
        1
        >>> etcsshconfig.global_lines[0].keyword
        'ProxyCommand'
        >>> etcsshconfig.global_lines[0].value
        'ssh -q -W %h:%p gateway.example.com'
        >>> 'Host_*' in etcsshconfig.host_lines
        True
        >>> etcsshconfig.host_lines['Host_*'][0].keyword
        'GSSAPIAuthentication'
        >>> etcsshconfig.host_lines['Host_*'][0].value
        'yes'
        >>> etcsshconfig.host_lines['Host_*'][1].keyword
        'ForwardX11Trusted'
        >>> etcsshconfig.host_lines['Host_*'][1].value
        'yes'
        >>> etcsshconfig.host_lines['Host_proxytest'][0].keyword
        'HostName'
        >>> etcsshconfig.host_lines['Host_proxytest'][0].value
        '192.168.122.2'
    """
    pass


@parser(Specs.ssh_config_d)
class EtcSshConfigD(SshClientConfig):
    """
    This Parser reads the files ``/etc/ssh/ssh_config.d/*``

    Sample output::

        # The options here are in the "Match final block" to be applied as the last
        # options and could be potentially overwritten by the user configuration
        Match final all
                # Follow system-wide Crypto Policy, if defined:
                Include /etc/crypto-policies/back-ends/openssh.config

                GSSAPIAuthentication yes

        # If this option is set to yes then remote X11 clients will have full access
        # to the original X11 display. As virtually no X11 client supports the untrusted
        # mode correctly we set this to yes.
                ForwardX11Trusted yes

        # Send locale-related environment variables
                SendEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
                SendEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
                SendEnv LC_IDENTIFICATION LC_ALL LANGUAGE
                SendEnv XMODIFIERS

        # Uncomment this if you want to use .local domain
        # Host *.local

    Attributes:

        global_lines (list): The list of site-wide configuration, as
            namedtuple('KeyValue', ['keyword', 'value', 'line']).
        host_lines (dict): The dict of all host-specific definitions, as
            {'Host_name': [namedtuple('KeyValue', ['keyword', 'value', 'line'])]}

    Examples:
        >>> etcsshconfigd.global_lines[1].keyword
        'Include'
        >>> etcsshconfigd.global_lines[1].value
        '/etc/crypto-policies/back-ends/openssh.config'
    """
    pass


@parser(Specs.ssh_foreman_config)
class ForemanSshConfig(SshClientConfig):
    """
    This parser reads the file ``/usr/share/foreman/.ssh/ssh_config``

    Sample output::

        #   ProxyCommand ssh -q -W %h:%p gateway.example.com
        #   RekeyLimit 1G 1h
        #
        # Uncomment this if you want to use .local domain
        # Host *.local
        #   CheckHostIP no
        ProxyCommand ssh -q -W %h:%p gateway.example.com

        Host *
            GSSAPIAuthentication yes
        # If this option is set to yes then remote X11 clients will have full access
        # to the original X11 display. As virtually no X11 client supports the untrusted
        # mode correctly we set this to yes.
            ForwardX11Trusted yes
        # Send locale-related environment variables
            SendEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
            SendEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
            SendEnv LC_IDENTIFICATION LC_ALL LANGUAGE
            SendEnv XMODIFIERS

        Host proxytest
            HostName 192.168.122.2

    Attributes:

        global_lines (list): The list of site-wide configuration, as
            namedtuple('KeyValue', ['keyword', 'value', 'line']).
        host_lines (dict): The dict of all host-specific definitions, as
            {'Host_name': [namedtuple('KeyValue', ['keyword', 'value', 'line'])]}

    Examples:
        >>> len(foremansshconfig.global_lines)
        1
        >>> foremansshconfig.global_lines[0].keyword
        'ProxyCommand'
        >>> foremansshconfig.global_lines[0].value
        'ssh -q -W %h:%p gateway.example.com'
        >>> 'Host_*' in foremansshconfig.host_lines
        True
        >>> foremansshconfig.host_lines['Host_*'][0].keyword
        'GSSAPIAuthentication'
        >>> foremansshconfig.host_lines['Host_*'][0].value
        'yes'
        >>> foremansshconfig.host_lines['Host_*'][1].keyword
        'ForwardX11Trusted'
        >>> foremansshconfig.host_lines['Host_*'][1].value
        'yes'
        >>> foremansshconfig.host_lines['Host_proxytest'][0].keyword
        'HostName'
        >>> foremansshconfig.host_lines['Host_proxytest'][0].value
        '192.168.122.2'
    """
    pass


@parser(Specs.ssh_foreman_proxy_config)
class ForemanProxySshConfig(SshClientConfig):
    """
    This parser reads the file ``/usr/share/foreman-proxy/.ssh/ssh_config``

    Sample output::

        #   ProxyCommand ssh -q -W %h:%p gateway.example.com
        #   RekeyLimit 1G 1h
        #
        # Uncomment this if you want to use .local domain
        # Host *.local
        #   CheckHostIP no
        ProxyCommand ssh -q -W %h:%p gateway.example.com

        Host *
            GSSAPIAuthentication yes
        # If this option is set to yes then remote X11 clients will have full access
        # to the original X11 display. As virtually no X11 client supports the untrusted
        # mode correctly we set this to yes.
            ForwardX11Trusted yes
        # Send locale-related environment variables
            SendEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
            SendEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
            SendEnv LC_IDENTIFICATION LC_ALL LANGUAGE
            SendEnv XMODIFIERS

        Host proxytest
            HostName 192.168.122.2

    Attributes:

        global_lines (list): The list of site-wide configuration, as
            namedtuple('KeyValue', ['keyword', 'value', 'line']).
        host_lines (dict): The dict of all host-specific definitions, as
            {'Host_name': [namedtuple('KeyValue', ['keyword', 'value', 'line'])]}

    Examples:
        >>> len(foreman_proxy_ssh_config.global_lines)
        1
        >>> foreman_proxy_ssh_config.global_lines[0].keyword
        'ProxyCommand'
        >>> foreman_proxy_ssh_config.global_lines[0].value
        'ssh -q -W %h:%p gateway.example.com'
        >>> 'Host_*' in foreman_proxy_ssh_config.host_lines
        True
        >>> foreman_proxy_ssh_config.host_lines['Host_*'][0].keyword
        'GSSAPIAuthentication'
        >>> foreman_proxy_ssh_config.host_lines['Host_*'][0].value
        'yes'
        >>> foreman_proxy_ssh_config.host_lines['Host_*'][1].keyword
        'ForwardX11Trusted'
        >>> foreman_proxy_ssh_config.host_lines['Host_*'][1].value
        'yes'
        >>> foreman_proxy_ssh_config.host_lines['Host_proxytest'][0].keyword
        'HostName'
        >>> foreman_proxy_ssh_config.host_lines['Host_proxytest'][0].value
        '192.168.122.2'
    """
    pass



import textwrap
import pytest

from insights.combiners.sssd_conf import SSSDConfAll
from insights.core.exceptions import SkipComponent
from insights.parsers.sssd_conf import SSSDConf
from insights.tests import context_wrap


def test_sssd_conf_all__domains_with_space():
    conf = textwrap.dedent(
        """
        [sssd]
        domains = a, b, c

        [domain/a]
        id_provider = ldap

        [domain/b]
        id_provider = ipa

        [domain/c]
        id_provider = ad
        """
    ).strip()

    sssd_conf = SSSDConf(context_wrap(conf, path="/etc/sssd/sssd.conf"))
    sssd_conf_all = SSSDConfAll(sssd_conf, [])

    assert sssd_conf_all.enabled_domains == ["a", "b", "c"]
    assert sssd_conf_all.domain_config("a") == {"id_provider": "ldap"}
    assert sssd_conf_all.domain_config("b") == {"id_provider": "ipa"}
    assert sssd_conf_all.domain_config("c") == {"id_provider": "ad"}


def test_sssd_conf_all__domains_without_space():
    conf = textwrap.dedent(
        """
        [sssd]
        domains = a,b,c

        [domain/a]
        id_provider = ldap

        [domain/b]
        id_provider = ipa

        [domain/c]
        id_provider = ad
        """
    ).strip()

    sssd_conf = SSSDConf(context_wrap(conf, path="/etc/sssd/sssd.conf"))
    sssd_conf_all = SSSDConfAll(sssd_conf, [])

    assert sssd_conf_all.enabled_domains == ["a", "b", "c"]
    assert sssd_conf_all.domain_config("a") == {"id_provider": "ldap"}
    assert sssd_conf_all.domain_config("b") == {"id_provider": "ipa"}
    assert sssd_conf_all.domain_config("c") == {"id_provider": "ad"}


def test_sssd_conf_all__enabled():
    conf = textwrap.dedent(
        """
        [sssd]

        [domain/a]
        enabled = true
        id_provider = ldap

        [domain/b]
        enabled = true
        id_provider = ipa

        [domain/c]
        id_provider = ad
        """
    ).strip()

    sssd_conf = SSSDConf(context_wrap(conf, path="/etc/sssd/sssd.conf"))
    sssd_conf_all = SSSDConfAll(sssd_conf, [])

    assert sssd_conf_all.enabled_domains == ["a", "b"]
    assert sssd_conf_all.domain_config("a") == {
        "enabled": "true",
        "id_provider": "ldap",
    }
    assert sssd_conf_all.domain_config("b") == {"enabled": "true", "id_provider": "ipa"}
    assert sssd_conf_all.domain_config("c") == {"id_provider": "ad"}


def test_sssd_conf_all__domains_and_enabled():
    conf = textwrap.dedent(
        """
        [sssd]
        domains = c

        [domain/a]
        enabled = true
        id_provider = ldap

        [domain/b]
        enabled = true
        id_provider = ipa

        [domain/c]
        id_provider = ad
        """
    ).strip()

    sssd_conf = SSSDConf(context_wrap(conf, path="/etc/sssd/sssd.conf"))
    sssd_conf_all = SSSDConfAll(sssd_conf, [])

    assert sssd_conf_all.enabled_domains == ["c", "a", "b"]
    assert sssd_conf_all.domain_config("a") == {
        "enabled": "true",
        "id_provider": "ldap",
    }
    assert sssd_conf_all.domain_config("b") == {"enabled": "true", "id_provider": "ipa"}
    assert sssd_conf_all.domain_config("c") == {"id_provider": "ad"}


def test_sssd_conf_all__conf_d():
    conf = textwrap.dedent(
        """
        [sssd]
        domains = a, b

        [domain/a]
        id_provider = ldap

        [domain/b]
        id_provider = ipa
        """
    ).strip()

    # overwrite domain a config
    # disable domain b
    conf_d_1 = textwrap.dedent(
        """
        [domain/a]
        id_provider = ipa

        [domain/b]
        enabled = false
        """
    ).strip()

    # overwrite domain a config
    # add domain c
    conf_d_2 = textwrap.dedent(
        """
        [domain/a]
        debug_level = 0xfff0

        [domain/c]
        enabled = true
        """
    ).strip()

    # dot file is ignored
    conf_d_dot = textwrap.dedent(
        """
        [domain/d]
        enabled = true
        """
    ).strip()

    sssd_conf = SSSDConf(context_wrap(conf, path="/etc/sssd/sssd.conf"))
    sssd_conf_d_1 = SSSDConf(context_wrap(conf_d_1, path="/etc/sssd/conf.d/1.conf"))
    sssd_conf_d_2 = SSSDConf(context_wrap(conf_d_2, path="/etc/sssd/conf.d/2.conf"))
    sssd_conf_d_dot = SSSDConf(
        context_wrap(conf_d_dot, path="/etc/sssd/conf.d/.3.conf")
    )
    sssd_conf_all = SSSDConfAll(
        sssd_conf, [sssd_conf_d_1, sssd_conf_d_2, sssd_conf_d_dot]
    )

    assert sssd_conf_all.enabled_domains == ["a", "c"]
    assert sssd_conf_all.domain_config("a") == {
        "id_provider": "ipa",
        "debug_level": "0xfff0",
    }
    assert sssd_conf_all.domain_config("b") == {
        "enabled": "false",
        "id_provider": "ipa",
    }
    assert sssd_conf_all.domain_config("c") == {"enabled": "true"}
    assert "domain/d" not in sssd_conf_all.config


def test_sssd_conf_all__missing_conf_d():
    conf = textwrap.dedent(
        """
        [sssd]
        domains = a, b

        [domain/a]
        id_provider = ldap

        [domain/b]
        id_provider = ipa
        """
    ).strip()

    sssd_conf = SSSDConf(context_wrap(conf, path="/etc/sssd/sssd.conf"))
    sssd_conf_all = SSSDConfAll(sssd_conf, None)

    assert sssd_conf_all.enabled_domains == ["a", "b"]
    assert sssd_conf_all.domain_config("a") == {"id_provider": "ldap"}
    assert sssd_conf_all.domain_config("b") == {"id_provider": "ipa"}


def test_sssd_conf_all__missing_conf__one_snippet():
    # set main config
    conf_d_1 = textwrap.dedent(
        """
        [sssd]
        domains = a, b

        [domain/a]
        id_provider = ldap

        [domain/b]
        id_provider = ipa
        """
    ).strip()

    sssd_conf_d_1 = SSSDConf(context_wrap(conf_d_1, path="/etc/sssd/conf.d/1.conf"))
    sssd_conf_all = SSSDConfAll(None, [sssd_conf_d_1])

    assert sssd_conf_all.enabled_domains == ["a", "b"]
    assert sssd_conf_all.domain_config("a") == {"id_provider": "ldap"}
    assert sssd_conf_all.domain_config("b") == {"id_provider": "ipa"}


def test_sssd_conf_all__missing_conf__two_snippets():
    # set main config
    conf_d_1 = textwrap.dedent(
        """
        [sssd]
        domains = a, b

        [domain/a]
        id_provider = ldap

        [domain/b]
        id_provider = ipa
        """
    ).strip()

    # overwrite domain a config
    conf_d_2 = textwrap.dedent(
        """
        [domain/a]
        id_provider = ipa
        """
    ).strip()

    sssd_conf_d_1 = SSSDConf(context_wrap(conf_d_1, path="/etc/sssd/conf.d/1.conf"))
    sssd_conf_d_2 = SSSDConf(context_wrap(conf_d_2, path="/etc/sssd/conf.d/2.conf"))
    sssd_conf_all = SSSDConfAll(None, [sssd_conf_d_1, sssd_conf_d_2])

    assert sssd_conf_all.enabled_domains == ["a", "b"]
    assert sssd_conf_all.domain_config("a") == {"id_provider": "ipa"}
    assert sssd_conf_all.domain_config("b") == {"id_provider": "ipa"}


def test_sssd_conf_all__missing_all():
    with pytest.raises(SkipComponent):
        SSSDConfAll(None, None)

    with pytest.raises(SkipComponent):
        SSSDConfAll(None, [])


def test_sssd_conf_all__domain_getters():
    conf = textwrap.dedent(
        """
        [domain/a]
        enabled = true
        id_provider = ldap
        """
    ).strip()

    sssd_conf = SSSDConf(context_wrap(conf, path="/etc/sssd/sssd.conf"))
    sssd_conf_all = SSSDConfAll(sssd_conf, [])

    assert sssd_conf_all.domain_section("a") == "domain/a"
    assert sssd_conf_all.domain_get("a", "enabled") == "true"
    assert sssd_conf_all.domain_get("a", "id_provider") == "ldap"
    assert sssd_conf_all.domain_get("a", "auth_provider", "ldap") == "ldap"
    assert sssd_conf_all.domain_get("a", "auth_provider") is None
    assert sssd_conf_all.domain_getboolean("a", "enabled") is True
    assert sssd_conf_all.domain_getboolean("a", "enumerate", False) is False
    assert sssd_conf_all.domain_getboolean("a", "enumerate") is None



import re


class FilePermissions(object):
    """
    Class for parsing `ls -l` line targeted at concrete file and handling parsed properties.

    It is useful for checking file permissions and owner.

    Attributes:
        perms_owner (str): Owner permissions, e.g. 'rwx'
        perms_group (str): Group permissions
        perms_other (str): Other permissions
        owner (str): Owner user name
        group (str): Owner group name
        path (str): Full path to file

    Note:
        This class does not support Access Control Lists (ACLs). If that is needed in the future,
        it would be preferable to create another class than extend this one.
        Advanced File Permissions - SUID, SGID and Sticky Bit - are not yet correctly parsed.
    """

    _PERMISSIONS_PATTERN = re.compile(r'''
        ^
        .([-rwxsS]{3})([-rwxsS]{3})([-rwxsS]{3})   # -rwxrwxrwx
        # -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
        # ^^^^^^^^^^
        # Valid characters are -rwxsS
        #   s == execute bit and sticky bit
        #   S == sticky bit without execute bit

        \S*                      # the character(s) after rwxrwxrwx for ACLs/xattrs
        # -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
        #           ^

        \s+\S+\s+                # the number of hardlinks and spaces around
        # -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
        #            ^^^

        ([^\s:]+)\s+([^\s:]+)    # owner, spaces, group
        # -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
        #               ^^^^^^^^^
        # Username and group name are strings without whitespace \s and without colon :.

        \s+\S+\s+                # size and spaces around
        # -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
        #                        ^^^^^^

        \S+\s+\S+                # month and day
        # -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
        #                              ^^^^^^

        \s+\S+\s+                # time/year and spaces around
        # -rw-------. 1 root root 4308 Apr 22 2009 /etc/ssh/sshd_config
        #                                    ^^^^^^
        # -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
        #                                    ^^^^^^^

        (.*)                     # file name or path
        # -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
        #                                           ^^^^^^^^^^^^^^^^^^^^
        # -rw-------. 1 root root 4308 Apr 22 15:57 file_name_without_path
        #                                           ^^^^^^^^^^^^^^^^^^^^^^

        $
        ''', re.VERBOSE)

    def __init__(self, line):
        """
        Args:
            line (str): A line from `ls -l /concrete/file` execution. Such as:
                        -rw-------. 1 root root 762 Sep 23 002 /etc/ssh/sshd_config
                        -rw-------. 1 root root 4308 Apr 22 15:57 /etc/ssh/sshd_config
                        -rw-r--r--. 1 root root 4179 Dec  1  2014 /boot/grub2/grub.cfg
        Raises:
            ValueError: If line is malformed
        """
        self.line = line
        r = self._PERMISSIONS_PATTERN.search(self.line)
        if r:
            (self.perms_owner, self.perms_group, self.perms_other,
             self.owner, self.group, self.path) = r.groups()
        else:
            raise ValueError('Invalid `ls -l` line "{}"'.format(self.line))

    @classmethod
    def from_dict(self, dirent):
        """
        Create a new FilePermissions object from the given dictionary.  This
        works with the FileListing parser class, which has already done the
        hard work of pulling many of these fields out.  We create an object
        with all the dictionary keys available as properties, and also split
        the ``perms`` string up into owner, group
        """
        # Check that we have at least as much data as the __init__ requires
        for k in ['perms', 'owner', 'group', 'name', 'dir']:
            if k not in dirent:
                raise ValueError("Need required key '{k}'".format(k=k))
        # Copy all values across
        for k in dirent:
            setattr(self, k, dirent[k])
        # Create perms parts
        self.perms_owner = self.perms[0:3]
        self.perms_group = self.perms[3:6]
        self.perms_other = self.perms[6:9]
        return self

    def owned_by(self, owner, also_check_group=False):
        """
        Checks if the specified user or user and group own the file.

        Args:
            owner (str): the user (or group) name for which we ask about ownership
            also_check_group (bool): if set to True, both user owner and group owner checked
                                if set to False, only user owner checked

        Returns:
            bool: True if owner of the file is the specified owner
        """
        if also_check_group:
            return self.owner == owner and self.group == owner
        else:
            return self.owner == owner

    def owner_can_read(self):
        """
        Checks if owner can read the file. Write and execute bits are not evaluated.

        Returns:
            bool: True if owner can read the file.
        """
        return 'r' in self.perms_owner

    def group_can_read(self):
        """
        Checks if group can read the file. Write and execute bits are not evaluated.

        Returns:
            bool: True if group can read the file.
        """
        return 'r' in self.perms_group

    def others_can_read(self):
        """
        Checks if 'others' can read the file. Write and execute bits are not evaluated. ('others' in
        the sense of unix permissions that know about user, group, others.)

        Returns:
            bool: True if 'others' can read the file.
        """
        return 'r' in self.perms_other

    def owner_can_only_read(self):
        """
        Checks if owner has read-only permissions for the file.
        Therefore, write and execute bits for owner must be unset and read bit must be set.

        Returns:
            bool: True if owner can only read the file.
        """
        return 'r--' == self.perms_owner

    def group_can_only_read(self):
        """
        Checks if group has read-only permissions for the file.
        Therefore, write and execute bits for group must be unset and read bit must be set.

        Returns:
            bool: True if group can only read the file.
        """
        return 'r--' == self.perms_group

    def others_can_only_read(self):
        """
        Checks if 'others' has read-only permissions for the file.
        Therefore, write and execute bits for 'others' must be unset and read bit must be set.
        ('others' in the sense of unix permissions that know about user, group, others.)

        Returns:
            bool: True if 'others' can only read the file.
        """
        return 'r--' == self.perms_other

    def owner_can_write(self):
        """
        Checks if owner can write the file. Read and execute bits are not evaluated.

        Returns:
            bool: True if owner can write the file.
        """
        return 'w' in self.perms_owner

    def group_can_write(self):
        """
        Checks if group can write the file. Read and execute bits are not evaluated.

        Returns:
            bool: True if group can write the file.
        """
        return 'w' in self.perms_group

    def others_can_write(self):
        """
        Checks if 'others' can write the file. Read and execute bits are not evaluated. ('others' in
        the sense of unix permissions that know about user, group, others.)

        Returns:
            bool: True if 'others' can write the file.
        """
        return 'w' in self.perms_other

    def only_root_can_read(self, root_group_can_read=True):
        """
        Checks if only root is allowed to read the file (and anyone else is
        forbidden from reading). Write and execute bits are not checked. The
        read bits for root user/group are not checked because root can
        read/write anything regardless of the read/write permissions.

        When called with ``root_group_can_read`` = ``True``:

        * owner must be root
        * and 'others' permissions must not contain read
        * and if group owner is not root, the 'group' permissions must not
          contain read

        Valid cases::

            rwxrwxrwx    owner   ownergroup
            -------------------------------
            ???-??-??    root    nonroot
            ??????-??    root    root
            r--r-----    root    root
            r--------    root    nonroot
            rwxrwx---    root    root
            rwxrwx-wx    root    root

        Specifically, these cases are NOT valid because the owner can chmod
        permissions and grant themselves permissions without root's
        knowledge::

            rwxrwxrwx    owner   ownergroup
            -------------------------------
            -??-??-??    nonroot nonroot
            -??r??-??    nonroot root
            ---------    nonroot nonroot

        When called with ``root_group_can_read`` = ``False``:

        * owner must be root
        * and 'group' and 'others' permissions must not contain read

        Valid cases::

            rwxrwxrwx    owner   ownergroup
            -------------------------------
            ???-??-??    root    ?
            r--------    root    root
            r--------    root    nonroot
            rwx-wx---    root    root
            rwx-wx---    root    nonroot
            rwx-wxrwx    root    nonroot

        Specifically, these cases are NOT valid because the owner can chmod
        permissions and grant themselves permissions without root's
        knowledge::

            rwxrwxrwx    owner   ownergroup
            -------------------------------
            -??-??-??    nonroot nonroot
            ---------    nonroot nonroot

        Args:
            root_group_can_read (bool): if set to True, this tests whether the
            'root' group can also read the file.

        Returns:
            bool: True if only root user (or optionally root group) can read
            the file.
        """

        requirements = True  # The final answer is progressively assembled in this variable.
        requirements &= self.owner == 'root'
        requirements &= not self.others_can_read()
        if root_group_can_read:
            if self.group != 'root':
                # if group is not root, group must not be able to read
                requirements &= not self.group_can_read()
        else:  # root_group_can_read == False
            requirements &= not self.group_can_read()
        return requirements

    def only_root_can_write(self, root_group_can_write=True):
        """
        Checks if only root is allowed to write the file (and anyone else is
        barred from writing). Read and execute bits are not checked. The
        write bits for root user/group are not checked because root can
        read/write anything regardless of the read/write permissions.

        When called with ``root_group_can_write`` = ``True``:

        * owner must be root
        * and 'others' permissions must not contain write
        * and if group owner is not root, the 'group' permissions must not contain write

        Valid cases::

            rwxrwxrwx    owner   ownergroup
            -------------------------------
            ????-??-?    root    nonroot
            ???????-?    root    root
            -w--w----    root    root
            -w-------    root    root
            rwxrwx---    root    root
            rwxrwxr-x    root    root

        Specifically, these cases are NOT valid because the owner can chmod
        permissions and grant themselves permissions without root's
        knowledge::

            rwxrwxrwx    owner   ownergroup
            -------------------------------
            ?-??-??-?    nonroot nonroot
            ?-??w??-?    nonroot root
            ---------    nonroot nonroot

        When called with ``root_group_can_write`` = ``False``:

        * owner must be root
        * and 'group' and 'others' permissions must not contain write

        Valid cases::

            rwxrwxrwx    owner   ownergroup
            -------------------------------
            ????-??-?    root    ?
            -w-------    root    root
            -w-------    root    nonroot
            rwxr-x---    root    root
            rwxr-x---    root    nonroot
            rwxr-xr-x    root    nonroot

        Specifically, these cases are NOT valid because the owner can chmod
        permissions and grant themselves permissions without root's
        knowledge::

            rwxrwxrwx    owner   ownergroup
            -------------------------------
            ?-??-??-?    nonroot nonroot
            ---------    nonroot nonroot

        Args:
            root_group_can_write (bool): if set to True, this tests whether
            'root' group can also write to the file.

        Returns:
            bool: True if only root user (or optionally root group) can write
            the file.
        """

        requirements = True  # The final answer is progressively assembled in this variable.
        requirements &= self.owner == 'root'
        requirements &= not self.others_can_write()
        if root_group_can_write:
            if self.group != 'root':
                # if group is not root, group must not be able to write
                requirements &= not self.group_can_write()
        else:  # root_group_can_write == False
            requirements &= not self.group_can_write()
        return requirements

    def all_zero(self):
        """
        Checks that all permissions are zero ('---------' in ls -l) - nobody but root can read,
        write, exec.

        Returns:
            bool: True if all permissions are zero ('---------')
        """
        _PERM_NOTHING = '---'
        return all((
            self.perms_owner == _PERM_NOTHING,
            self.perms_group == _PERM_NOTHING,
            self.perms_other == _PERM_NOTHING,
        ))

    def __repr__(self):
        return 'FilePermissions(' + self.path + ')'



import doctest

from insights.parsers import containers_inspect
from insights.parsers.containers_inspect import ContainersInspect
from insights.tests import context_wrap


CONTAINERS_INSPECT = """
[{"Id": "aeaea3ead527", "engine": "podman", "Image": "538460c14d75dee1504e56ad8ddb7fe039093b1530ef8f90442a454b9aa3dc8b", "Config|Cmd": ["sleep", "1000000"], "HostConfig|Privileged": false}, {"Id": "28fb57be8bb2", "engine": "podman", "Image": "538460c14d75dee1504e56ad8ddb7fe039093b1530ef8f90442a454b9aa3dc8b", "Config|Cmd": ["sleep", "1000000"], "HostConfig|Privileged": true}, {"Id": "c7efee959ea8", "engine": "docker", "Image": "acf3e09a39c95d354539b6591298be0b0814f5d74e95e722863241192b9a079b", "Config|Cmd": ["sleep", "1000000"], "HostConfig|Privileged": true}]
""".strip()


def test_containers_inspect():
    containers_inspect_result = ContainersInspect(context_wrap(CONTAINERS_INSPECT))
    assert containers_inspect_result.data[0]["Id"] == "aeaea3ead527"
    assert containers_inspect_result.data[0]["HostConfig|Privileged"] is False
    assert containers_inspect_result.data[0]["Config|Cmd"] == ["sleep", "1000000"]


def test_doc_examples():
    env = {
        'inspect_containers': ContainersInspect(context_wrap(CONTAINERS_INSPECT))
    }
    failed, total = doctest.testmod(containers_inspect, globs=env)
    assert failed == 0



"""
NormalMD5 - md5 checksums of specified binary or library files
==============================================================

Module for processing output of the ``md5sum`` command.

The name and md5 checksums of the specified file are stored as attributes.
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.md5chk_files)
class NormalMD5(CommandParser):
    """
    Class to parse the ``md5sum`` command information.

    The output of this command contains two fields, the first is the
    md5 checksum and the second is the file name.

    Sample output of the ``md5sum`` command::

        d1e6613cfb62d3f111db7bdda39ac821  /usr/lib64/libsoftokn3.so

    Examples:

        >>> type(md5info)
        <class 'insights.parsers.md5check.NormalMD5'>
        >>> md5info.filename
        '/etc/localtime'
        >>> md5info.md5sum
        '7d4855248419b8a3ce6616bbc0e58301'

    Attributes:
        filename (str): Filename for which the MD5 checksum was computed.
        md5sum (str): MD5 checksum value.
    """

    def parse_content(self, content):
        if len(content) != 1:
            raise ParseException("Incorrect length for input {length}.".format(length=len(content)))
        for line in content:
            self.md5sum, self.filename = content[-1].strip().split(None, 1)
            if len(self.md5sum) != 32:
                raise ParseException("Invalid MD5sum value {length}.".format(length=self.md5sum))



"""
TomcatVirtualDirContextCombined - check VirtualDirContext option used in Tomcat
===============================================================================

This combiner provides information about whether a VirtualDirContext is used in config files in
both default locations or location derived from running Tomcat process.

Examples::

    >>> shared[TomcatVirtualDirContextCombined].data
    {'/usr/share/tomcat/conf/server.xml':
     ['    <Resources className="org.apache.naming.resources.VirtualDirContext"'],
     }
"""
from ..core.plugins import combiner
from ..parsers.tomcat_virtual_dir_context import TomcatVirtualDirContextFallback, \
    TomcatVirtualDirContextTargeted


@combiner([TomcatVirtualDirContextFallback, TomcatVirtualDirContextTargeted])
class TomcatVirtualDirContextCombined(object):
    """
    Combiner for VirtualDirContext usage in Tomcat config files.
    """
    def __init__(self, tomcat_vdc_fallback, tomcat_vdc_targeted):
        self.data = {}
        fallback = tomcat_vdc_fallback if tomcat_vdc_fallback else None  # Returns one parser
        targeted = tomcat_vdc_targeted if tomcat_vdc_targeted else []  # Returns list of parsers
        for parser in [fallback] + targeted:
            if parser:
                for key in parser.data:
                    if key not in self.data:
                        self.data[key] = []
                    self.data[key] += parser.data[key]

        # Deduplicate lines
        for key, value in self.data.items():
            self.data[key] = sorted(set(value))



"""
Custom datasources for lpstat information
"""
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by lpstat datasources """

    lpstat_v = simple_command("/usr/bin/lpstat -v")
    """ Returns the output of command ``/usr/bin/lpstat -v`` """
    lpstat_o = simple_command("/usr/bin/lpstat -o")
    """ Returns the output of command ``/usr/bin/lpstat -o`` """


@datasource(LocalSpecs.lpstat_v, HostContext)
def lpstat_protocol_printers_info(broker):
    """
    This datasource provides the not-sensitive information collected
    from ``/usr/bin/lpstat -v``.

    Typical content of ``/usr/bin/lpstat -v`` file is::

        "device for test_printer1: ipp://cups.test.com/printers/test_printer1"

    Returns:
        DatasourceProvider: Returns the collected content containing non-sensitive information

    Raises:
        SkipComponent: When the filter/path does not exist or any exception occurs.
    """
    try:
        content = broker[LocalSpecs.lpstat_v].content
        result = []
        for line in content:
            if "device for " in line:
                "Remove printer address information"
                result.append(line.split("://", 1)[0] if '://' in line else line)
        if result:
            return DatasourceProvider(content="\n".join(result), relative_path='insights_commands/lpstat_-v')
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))
    raise SkipComponent


@datasource(LocalSpecs.lpstat_o, HostContext)
def lpstat_queued_jobs_count(broker):
    """
    This datasource provides the count of all queued job.

    Typical content of ``/usr/bin/lpstat -o`` file is::

        Cups-PDF-1802           root          265443328   Tue 05 Sep 2023 02:21:19 PM CST
        Cups-PDF-1803           root          265443328   Tue 05 Sep 2023 02:21:21 PM CST
        Cups-PDF-1804           root          265443328   Tue 05 Sep 2023 02:21:22 PM CST

    Sample data returned::

        3

    Returns:
        DatasourceProvider: Returns the collected content containing the count of all queued jobs.

    Raises:
        SkipComponent: When there is not any content.
    """

    content = broker[LocalSpecs.lpstat_o].content
    if content:
        cnt = 0
        bad_lines = ["no such file or directory",
                     "not a directory",
                     "command not found",
                     "no module named",
                     "no files found for",
                     "missing dependencies:",
                     "Bad file descriptor"]
        for line in content:
            if not any(key in line for key in bad_lines):
                cnt += 1
        if cnt:
            return DatasourceProvider(content="{0}".format(cnt), relative_path='insights_commands/lpstat_-o_jobs_count')
    raise SkipComponent



from insights.parsers.fc_match import FCMatch
from insights.tests import context_wrap

FCMATCH_V1 = """
Pattern has 2 elts (size 16)
    family: "DejaVu Sans"(s)
    fontformat: "TrueType"(w)

Pattern has 2 elts (size 16)
    family: "DejaVu Sans"(s)
    fontformat: "TrueType"(w)

Pattern has 2 elts (size 16)
    family: "DejaVu Sans"(s)
    fontformat: "TrueType"(w)

Pattern has 2 elts (size 16)
    family: "Nimbus Sans L"(s)
    fontformat: "Type 1"(s)

Pattern has 2 elts (size 16)
    family: "Standard Symbols L"(s)
    fontformat: "Type 1"(s)
"""


def test_fcmatch_v1():
    fc_match = FCMatch(context_wrap(FCMATCH_V1))
    assert fc_match.data == [{'fontformat': '"TrueType"(w)', 'family': '"DejaVu Sans"(s)'},
                             {'fontformat': '"Type 1"(s)', 'family': '"Nimbus Sans L"(s)'},
                             {'fontformat': '"Type 1"(s)', 'family': '"Standard Symbols L"(s)'}]
    assert fc_match[0] == {'fontformat': '"TrueType"(w)', 'family': '"DejaVu Sans"(s)'}
    for item in fc_match:
        if item["family"] == '"DejaVu Sans"(s)':
            assert item["fontformat"] == '"TrueType"(w)'
